@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	10 October 1999
+ * Last Edited:	1 November 1999
  *
  * Copyright 1999 by the University of Washington
  *
@@ -393,8 +393,7 @@ long server_input_wait (long seconds)
   tmo.tv_sec = seconds; tmo.tv_usec = 0;
   return select (1,&rfd,0,&efd,&tmo) ? LONGT : NIL;
 }
-
-
+
 /* Server log in
  * Accepts: user name string
  *	    password string
@@ -413,11 +412,24 @@ long server_login (char *user,char *pwd,int argc,char *argv[])
 	    tcp_clienthost ());
 				/* validate with case-independence */
   else if ((logtry > 0) && ((pw = getpwnam (strcpy (usr,user))) ||
-			    (pw = getpwnam (lcase (usr)))) &&
-	   ((pw = checkpw (pw,pwd,argc,argv)) ||
-	    ((*pwd == ' ') && (pw = getpwnam (usr)) &&
-	     (pw = checkpw (pw,pwd + 1,argc,argv)))))
-    return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+			    (pw = getpwnam (lcase (usr))))) {
+    if (auth_md5.server) {	/* if using CRAM-MD5 authentication */
+      char *p = auth_md5_pwd (pw->pw_name);
+      if (p) {			/* verify password */
+	if (strcmp (p,pwd) && strcmp (p,pwd+1)) pw = NIL;
+	memset (p,0,strlen (p));/* erase sensitive information */
+	fs_give ((void **) &p);	/* flush erased password */
+	if (pw) return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+      }
+      else syslog (LOG_ERR|LOG_AUTH,
+		   "Login failed: %s has no CRAM-MD5 password, host=%.80s",
+		   pw->pw_name,tcp_clienthost ());
+    }
+				/* ordinary password authentication */
+    else if ((pw = checkpw (pw,pwd,argc,argv)) ||
+	     ((*pwd == ' ') && (pw = checkpw (getpwnam(usr),pwd+1,argc,argv))))
+      return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+  }
   s = (logtry-- > 0) ? "Login failure" : "Excessive login attempts";
 				/* note the failure in the syslog */
   syslog (LOG_INFO,"%s user=%.80s host=%.80s",s,user,tcp_clienthost ());
@@ -489,7 +501,7 @@ long env_init (char *user,char *home)
   char *s,tmp[MAILTMPLEN];
   if (myUserName) fatal ("env_init called twice!");
 				/* set up user name */
-  myUserName = cpystr ((char *) (user ? user : anonymous_user));
+  myUserName = cpystr (user ? user : (char *) anonymous_user);
   if (!anonymousHome) anonymousHome = cpystr (ANONYMOUSHOME);
   if (user) {			/* remember user name and home directory */
     nslist[0] = &nshome,nslist[1] = &nsamigaother,nslist[2] = &nsshared;
@@ -530,7 +542,7 @@ char *myusername_full (unsigned long *flags)
   if (!myUserName) {		/* get user name if don't have it yet */
     struct passwd *pw;
     unsigned long euid = geteuid ();
-    char *s = (char *) getlogin ();
+    char *s = (char *) (euid ? getlogin () : NIL);
 				/* look up getlogin() user name or EUID */
     if (!((s && *s && (pw = getpwnam (s)) && (pw->pw_uid == euid)) ||
 	  (pw = getpwuid (euid)))) fatal ("Unable to look up user name");
@@ -897,7 +909,8 @@ int lock_work (char *lock,void *sb,int op,long *pid)
 	(lsb.st_ino == fsb.st_ino) && (fsb.st_nlink == 1)) break;
     close (fd);			/* lock not right, drop fd and try again */
   }
-  chmod (lock,lock_protection);	/* make sure mode OK (don't use fchmod()) */
+				/* make sure mode OK (don't use fchmod()) */
+  chmod (lock,(int) lock_protection);
   return fd;			/* success */
 }
 
