@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	10 October 1999
+ * Last Edited:	17 October 1999
  *
  * Copyright 1999 by the University of Washington
  *
@@ -417,8 +417,7 @@ long server_input_wait (long seconds)
   tmo.tv_sec = seconds; tmo.tv_usec = 0;
   return select (1,&rfd,0,&efd,&tmo) ? LONGT : NIL;
 }
-
-
+
 /* Server log in
  * Accepts: user name string
  *	    password string
@@ -437,11 +436,24 @@ long server_login (char *user,char *pwd,int argc,char *argv[])
 	    tcp_clienthost ());
 				/* validate with case-independence */
   else if ((logtry > 0) && ((pw = getpwnam (strcpy (usr,user))) ||
-			    (pw = getpwnam (lcase (usr)))) &&
-	   ((pw = checkpw (pw,pwd,argc,argv)) ||
-	    ((*pwd == ' ') && (pw = getpwnam (usr)) &&
-	     (pw = checkpw (pw,pwd + 1,argc,argv)))))
-    return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+			    (pw = getpwnam (lcase (usr))))) {
+    if (auth_md5.server) {	/* if using CRAM-MD5 authentication */
+      char *p = auth_md5_pwd (pw->pw_name);
+      if (p) {			/* verify password */
+	if (strcmp (p,pwd) && strcmp (p,pwd+1)) pw = NIL;
+	memset (p,0,strlen (p));/* erase sensitive information */
+	fs_give ((void **) &p);	/* flush erased password */
+	if (pw) return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+      }
+      else syslog (LOG_ERR|LOG_AUTH,
+		   "Login failed: %s has no CRAM-MD5 password, host=%.80s",
+		   pw->pw_name,tcp_clienthost ());
+    }
+				/* ordinary password authentication */
+    else if ((pw = checkpw (pw,pwd,argc,argv)) ||
+	     ((*pwd == ' ') && (pw = checkpw (getpwnam(usr),pwd+1,argc,argv))))
+      return pw_login (pw,pw->pw_name,pw->pw_dir,argc,argv);
+  }
   s = (logtry-- > 0) ? "Login failure" : "Excessive login attempts";
 				/* note the failure in the syslog */
   syslog (LOG_INFO,"%s user=%.80s host=%.80s",s,user,tcp_clienthost ());
@@ -513,7 +525,7 @@ long env_init (char *user,char *home)
   char *s,tmp[MAILTMPLEN];
   if (myUserName) fatal ("env_init called twice!");
 				/* myUserName must be set before dorc() call */
-  myUserName = cpystr ((char *) (user ? user : anonymous_user));
+  myUserName = cpystr (user ? user : (char *) anonymous_user);
 				/* do systemwide configuration */
   dorc ("/etc/c-client.cf",NIL);
   if (!anonymousHome) anonymousHome = cpystr (ANONYMOUSHOME);
@@ -526,6 +538,8 @@ long env_init (char *user,char *home)
 	sprintf (tmp,"%s/INBOX",myHomeDir = cpystr (s));
 	sysInbox = cpystr (tmp);/* set black box values in their place */
 	blackBox = T;
+				/* mbox meaningless if black box */
+	mail_parameters (NIL,DISABLE_DRIVER,(void *) "mbox");
       }
     }
     if (blackBox)		/* black box? */
@@ -576,7 +590,7 @@ char *myusername_full (unsigned long *flags)
   if (!myUserName) {		/* get user name if don't have it yet */
     struct passwd *pw;
     unsigned long euid = geteuid ();
-    char *s = (char *) getlogin ();
+    char *s = (char *) (euid ? getlogin () : NIL);
 				/* look up getlogin() user name or EUID */
     if (!((s && *s && (pw = getpwnam (s)) && (pw->pw_uid == euid)) ||
 	  (pw = getpwuid (euid)))) fatal ("Unable to look up user name");
@@ -967,7 +981,8 @@ int lock_work (char *lock,void *sb,int op,long *pid)
 	(lsb.st_ino == fsb.st_ino) && (fsb.st_nlink == 1)) break;
     close (fd);			/* lock not right, drop fd and try again */
   }
-  chmod (lock,lock_protection);	/* make sure mode OK (don't use fchmod()) */
+				/* make sure mode OK (don't use fchmod()) */
+  chmod (lock,(int) lock_protection);
   return fd;			/* success */
 }
 
