@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: attach.c,v 4.61 1998/12/16 03:24:01 mikes Exp $";
+static char rcsid[] = "$Id: attach.c,v 4.63 1999/05/27 23:09:27 mikes Exp $";
 #endif
 /*
  * Program:	Routines to support attachments in the Pine composer 
@@ -136,23 +136,26 @@ char *fn, *sz, *cmnt;
 	  case (CTRL|'I') :
 	    if(i == 2){
 		char *fname, *p;
-		int   l = NLINE;
-
+		int   dirlen, l = NLINE;  /* length of buffer space for bfn */
+	       
 		bfn[0] = '\0';
 		if(*fn && (p = strrchr(fn, C_FILESEP))){
 		    fname = p + 1;
 		    l -= fname - fn;
+		    dirlen = p - fn;
 		    if(p == fn)
 		      strcpy(bfn, S_FILESEP);
 #ifdef	DOS
 		    else if(fn[0] == C_FILESEP
 			    || (isalpha((unsigned char)fn[0])
 				&& fn[1] == ':')){
+		        if(fn[1] == ':' && p == fn+2)
+			  dirlen = fname - fn;
 #else
 		    else if (fn[0] == C_FILESEP || fn[0] == '~') {
 #endif
-			strncpy(bfn, fn, p - fn);
-			bfn[p-fn] = '\0';
+			strncpy(bfn, fn, dirlen);
+			bfn[dirlen] = '\0';
 		    }
 		    else
 		      sprintf(bfn, "%s%c%.*s", 
@@ -1025,7 +1028,7 @@ process_tag:					/* enclosed in []         */
 	/*
 	 * not in comment or label name? done. 
 	 */
-	if(c == ',' && (level != CMMNT && !lbln))
+	if(c == ',' && (level != TAG && level != CMMNT && !lbln))
 	  break;				/* put offset past ',' */
     }
 
