@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.82 1998/09/16 00:18:45 hubert Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.83 1999/10/01 23:20:59 mikes Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -1423,7 +1423,7 @@ int   dirlen, fnlen, flags;
 
 	    while(!i){
 
-		switch(readpattern("File name to find")){
+		switch(readpattern("File name to find", FALSE)){
 		  case HELPCH:
 		    emlwrite("\007No help yet!", NULL);
 /* remove break and sleep after help text is installed */
