@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.88 1998/12/29 18:20:20 hubert Exp $";
+static char rcsid[] = "$Id: display.c,v 4.97 1999/09/01 19:24:33 hubert Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: display.c,v 4.88 1998/12/29 18:20:20 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-1999 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -1095,6 +1095,15 @@ clearcursor()
     ttrow = ttcol = FARAWAY;
 }
 
+get_cursor(row, col)
+    int *row, *col;
+{
+    if(row)
+      *row = ttrow;
+    if(col)
+      *col = ttcol;
+}
+
 
 /*
  * Erase the message line. This is a special routine because the message line
@@ -1383,10 +1392,25 @@ EXTRAKEYS *extras;
 	movecursor(term.t_nrow - term.t_mrow, plen + b - buf);
 	(*term.t_flush)();
 
-
+#ifdef	MOUSE
+	mouse_in_content(KEY_MOUSE, -1, -1, 0x5, 0);
+	register_mfunc(mouse_in_content, 
+		       term.t_nrow - term.t_mrow, plen,
+		       term.t_nrow - term.t_mrow, plen + maxl);
+#endif
+#ifdef	_WINDOWS
+	mswin_allowpaste(MSWIN_PASTE_LINE);
+#endif
 	while((c = GetKey()) == NODATA)
 	  ;
 
+#ifdef	MOUSE
+	clear_mfunc(mouse_in_content);
+#endif
+#ifdef	_WINDOWS
+	mswin_allowpaste(MSWIN_PASTE_DISABLE);
+#endif
+
 	switch(c = normalize_cmd(c, rfkm, 1)){
 	  case (CTRL|'A') :			/* CTRL-A beginning     */
 	  case KEY_HOME :
@@ -1488,6 +1512,39 @@ EXTRAKEYS *extras;
 	    return_val = changed;
 	    goto ret;
 
+#ifdef	MOUSE
+	  case KEY_MOUSE :
+	    {
+	      MOUSEPRESS mp;
+
+	      mouse_get_last (NULL, &mp);
+
+	      /* The clicked line have anything special on it? */
+	      switch(mp.button){
+		case M_BUTTON_LEFT :			/* position cursor */
+		  mp.col -= plen;			/* normalize column */
+		  if(mp.col >= 0 && mp.col <= strlen(buf))
+		    b = buf + mp.col;
+
+		  break;
+
+		case M_BUTTON_RIGHT :
+#ifdef	_WINDOWS
+		  mswin_allowpaste(MSWIN_PASTE_LINE);
+		  mswin_paste_popup();
+		  mswin_allowpaste(MSWIN_PASTE_DISABLE);
+		  break;
+#endif
+
+		case M_BUTTON_MIDDLE :			/* NO-OP for now */
+		default:				/* just ignore */
+		  break;
+	      }
+	    }
+
+	    continue;
+#endif
+
 	  default : 
 	    if(strlen(buf) >= maxl){		/* contain the text      */
 		(*term.t_beep)();
@@ -2014,9 +2071,8 @@ int  row, col;
 	    ResizeHeader();
 	}
 	else{
-	    lchange(WFHARD);                   /* set update flags... */
-	    curwp->w_flag |= WFMODE;           /* and modeline so we  */
-	    refresh(0, 1);                     /* redraw the whole enchilada. */
+	    curwp->w_flag |= (WFHARD | WFMODE);
+	    refresh(0, 1);                     /* redraw whole enchilada. */
 	    update();                          /* do it */
 	}
     }
@@ -2040,19 +2096,31 @@ showCompTitle()
     if(Pmaster){
 	register char *bufp;
 	extern   char *pico_anchor;
+	COLOR_PAIR *lastc = NULL;
 
 	if((bufp = pico_anchor) == NULL)
 	  return(1);
-
+	
 	movecursor(COMPOSER_TITLE_LINE, 0);
-	(*term.t_rev)(1);   
+	if (Pmaster->colors && Pmaster->colors->tbcp &&
+	    pico_is_good_colorpair(Pmaster->colors->tbcp)){
+	  lastc = pico_get_cur_color();
+	  (void)pico_set_colorp(Pmaster->colors->tbcp, PSC_NONE);
+	}
+	else
+	  (*term.t_rev)(1);   
 	while (ttcol < term.t_ncol)
 	  if(*bufp != '\0')
 	    pputc(*bufp++, 1);
           else
 	    pputc(' ', 1);
 
-	(*term.t_rev)(0);
+	if (lastc){
+	  (void)pico_set_colorp(lastc, PSC_NONE);
+	  free_color_pair(&lastc);
+	}
+	else
+	  (*term.t_rev)(0);
 	movecursor(COMPOSER_TITLE_LINE + 1, 0);
 	peeol();
     }
@@ -2277,6 +2345,19 @@ int      key;
     register int  i = 0;
     register int  j = 0;
     register int  l;
+    COLOR_PAIR *lastc = NULL;
+    COLOR_PAIR *kncp = NULL;
+    COLOR_PAIR *klcp = NULL;
+
+    if (Pmaster && Pmaster->colors){
+      if(pico_is_good_colorpair(Pmaster->colors->klcp))
+        klcp = Pmaster->colors->klcp;
+
+      if(klcp && pico_is_good_colorpair(Pmaster->colors->kncp))
+        kncp = Pmaster->colors->kncp;
+
+      lastc = pico_get_cur_color();
+    }
 
     l = strlen(pmt);
     while(1){
@@ -2300,20 +2381,28 @@ int      key;
     }
 
     movecursor(line, column+i);
+    if(klcp) (void)pico_set_colorp(klcp, PSC_NONE);
     buf = &pmt[j];
     do{
 	if(*buf == key){
 	    buf++;
-	    (*term.t_rev)(1);
+	    kncp ? (void)pico_set_colorp(kncp, PSC_NONE)
+		 : (void)(*term.t_rev)(1);
 	    pputc(*buf, 1);
-	    (*term.t_rev)(0);
+	    kncp ? (void)pico_set_colorp(klcp, PSC_NONE)
+		 : (void)(*term.t_rev)(0);
 	}
 	else{
 	    pputc(*buf, 0);
 	}
     }    
     while(*++buf != '\0');
+
     peeol();
+    if (lastc){
+      (void)pico_set_colorp(lastc, PSC_NONE);
+      free_color_pair(&lastc);
+    }
     (*term.t_flush)();
 }
 
@@ -2390,7 +2479,11 @@ KEYMENU *keymenu;
 				   : keymenu[index].name[0],
 			     nbuf, invert_label,
 			     term.t_nrow - 1 + row, (slot * tspace),
-			     strlen(nbuf));
+			     strlen(nbuf),
+			     (Pmaster && Pmaster->colors) 
+			       ? Pmaster->colors->kncp: NULL,
+			     (Pmaster && Pmaster->colors) 
+			       ? Pmaster->colors->klcp: NULL);
 #endif
 
 		n = nspace[slot];
@@ -2417,7 +2510,7 @@ KEYMENU *keymenu;
 		  *obufp++ = ' ';
 
 #ifdef	MOUSE
-		register_key(index, NODATA, "", NULL, 0, 0, 0);
+		register_key(index, NODATA, "", NULL, 0, 0, 0, NULL, NULL);
 #endif
 	    }
 
