@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.68 1998/11/11 00:51:05 hubert Exp $
+ * $Id: efunc.h,v 4.80 1999/10/01 23:21:07 mikes Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -20,7 +20,7 @@
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-1999 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -142,6 +142,7 @@ extern	int pinsert PROTO((CELL));
 extern	int pdel PROTO((void));
 extern	void wstripe PROTO((int, int, char *, int));
 extern	int wkeyhelp PROTO((KEYMENU *));
+extern	int get_cursor PROTO((int *, int *));
 
 /* file.c */
 extern	int fileread PROTO((int, int));
@@ -178,6 +179,13 @@ extern	int fremove PROTO((int));
 
 /* os.c */
 extern	int Raw PROTO((int));
+extern	void StartInverse PROTO((void));
+extern	void EndInverse PROTO((void));
+extern	int InverseState PROTO((void));
+extern	int StartBold PROTO((void));
+extern	void EndBold PROTO((void));
+extern	void StartUnderline PROTO((void));
+extern	void EndUnderline PROTO((void));
 extern	void xonxoff_proc PROTO((int));
 extern	void crlf_proc PROTO((int));
 extern	void intr_proc PROTO((int));
@@ -233,6 +241,34 @@ extern	int struncmp PROTO((char *, char *, int));
 extern	int chkptinit PROTO((char *, int));
 extern	void set_collation PROTO((void));
 extern	int (*pcollator)();
+extern  COLOR_PAIR *new_color_pair PROTO((char *, char *));
+extern	COLOR_PAIR *pico_get_cur_color PROTO((void));
+extern	COLOR_PAIR *pico_get_rev_color PROTO((void));
+extern	void free_color_pair PROTO((COLOR_PAIR **));
+extern	void pico_nfcolor PROTO((char *));
+extern	void pico_nbcolor PROTO((char *));
+extern	void pico_rfcolor PROTO((char *));
+extern	void pico_rbcolor PROTO((char *));
+extern	int pico_set_fg_color PROTO((char *));
+extern	int pico_set_bg_color PROTO((char *));
+extern	COLOR_PAIR *pico_set_colorp PROTO((COLOR_PAIR *, int));
+extern	COLOR_PAIR *pico_set_colors PROTO((char *, char *, int));
+extern	int pico_is_good_color PROTO((char *));
+extern	int pico_is_good_colorpair PROTO((COLOR_PAIR *));
+extern	void pico_set_nfg_color PROTO((void));
+extern	void pico_set_nbg_color PROTO((void));
+extern	void pico_set_normal_color PROTO((void));
+extern	int pico_usingcolor PROTO((void));
+extern	char *color_to_asciirgb PROTO((char *));
+extern	int pico_count_in_color_table PROTO((void));
+/* these color functions aren't needed for Windows */
+extern	int pico_hascolor PROTO((void));
+extern	void pico_toggle_color PROTO((int));
+extern	unsigned pico_get_color_options PROTO((void));
+extern	void pico_set_color_options PROTO((unsigned));
+extern	void pico_endcolor PROTO((void));
+extern	char *pico_get_last_fg_color PROTO((void));
+extern	char *pico_get_last_bg_color PROTO((void));
 #ifdef	MOUSE
 extern	unsigned long pico_mouse PROTO((unsigned, int, int));
 #endif
@@ -249,6 +285,10 @@ extern	int ctrlg PROTO((int, int));
 extern	int rdonly PROTO((void));
 extern	int pico_help PROTO((char **, char *, int));
 extern	int zotedit PROTO((void));
+#ifdef	_WINDOWS
+int	composer_file_drop PROTO((int, int, char *));
+int	pico_cursor PROTO((int, long));
+#endif
 
 /* random.c */
 extern	int showcpos PROTO((int, int));
@@ -258,6 +298,7 @@ extern	int forwdel PROTO((int, int));
 extern	int backdel PROTO((int, int));
 extern	int killtext PROTO((int, int));
 extern	int yank PROTO((int, int));
+extern	char *colorx PROTO((int));
 
 /* region.c */
 extern	int killregion PROTO((int, int));
@@ -266,7 +307,7 @@ extern	int markregion PROTO((int));
 
 /* search.c */
 extern	int forwsearch PROTO((int, int));
-extern	int readpattern PROTO((char *));
+extern	int readpattern PROTO((char *, int));
 extern	int forscan PROTO((int *, char *, LINE *, int, int));
 
 /* spell.c */
