@@ -1,5 +1,5 @@
 /*
- * $Id: estruct.h,v 4.40 1998/08/05 18:20:08 mikes Exp $
+ * $Id: estruct.h,v 4.50 1999/10/04 20:02:31 mikes Exp $
  *
  * Program:	Struct and preprocessor definitions
  *
@@ -20,7 +20,7 @@
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-1999 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -106,6 +106,7 @@
 #define CFCPCN  0x0001                  /* Last command was C-P, C-N    */
 #define CFKILL  0x0002                  /* Last command was a kill      */
 #define CFFILL  0x0004                  /* Last command was a kill      */
+#define	CFSRCH	0x0008			/* Last command was WhereIs	*/
 
 #define BACKSPACE	'\b'     	/* backspace character  */
 #define TAB		'\t'            /* tab character        */
@@ -283,6 +284,36 @@ typedef	struct CELL {
 	unsigned int a : 8;		/* Its attributes               */
 } CELL;
 
+#define RGBLEN 11
+#define MAXCOLORLEN 11			/* longest string a color can be */
+
+typedef struct COLOR_PAIR {
+	char fg[MAXCOLORLEN+1];
+	char bg[MAXCOLORLEN+1];
+} COLOR_PAIR;
+
+#define COL_BLACK	0
+#define COL_RED		1
+#define COL_GREEN	2
+#define COL_YELLOW	3
+#define COL_BLUE	4
+#define COL_MAGENTA	5
+#define COL_CYAN	6
+#define COL_WHITE	7
+
+#define DEFAULT_NORM_FORE	COL_BLACK
+#define DEFAULT_NORM_BACK	COL_CYAN
+
+/* flags for pico_set_color() */
+#define PSC_NONE	0x0
+#define PSC_NORM	0x1
+#define PSC_REV		0x2
+#define PSC_RET		0x4	/* return an allocated copy of previous color */
+
+/* flags for color_options */
+#define COLOR_ANSI8_OPT		0x01
+#define COLOR_ANSI16_OPT	0x02
+
 
 /*
  * All text is kept in circularly linked lists of "LINE" structures. These
