@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.69 1999/01/29 19:59:05 hubert Exp $";
+static char rcsid[] = "$Id: file.c,v 4.70 1999/03/23 01:25:05 jfrankli Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -209,23 +209,26 @@ int f, n;
 	      case (CTRL|'I') :
 		{
 		    char *fn, *p;
-		    int   l = NLINE;
+		    int   dirlen, l = NLINE;
 
 		    dir[0] = '\0';
 		    if(*fname && (p = strrchr(fname, C_FILESEP))){
 			fn = p + 1;
 			l -= fn - fname;
+			dirlen = p - fname;
 			if(p == fname)
 			  strcpy(dir, S_FILESEP);
 #ifdef	DOS
 			else if(fname[0] == C_FILESEP
 				 || (isalpha((unsigned char)fname[0])
 				     && fname[1] == ':')){
+			  if(fname[1] == ':' && p == fname+2)
+			  dirlen = fn - fname;
 #else
 			else if (fname[0] == C_FILESEP || fname[0] == '~') {
 #endif
-			    strncpy(dir, fname, p - fname);
-			    dir[p-fname] = '\0';
+			    strncpy(dir, fname, dirlen);
+			    dir[dirlen] = '\0';
 			}
 			else
 			  sprintf(dir, "%s%c%.*s", 
