@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: line.c,v 4.20 1998/04/02 05:59:44 mikes Exp $";
+static char rcsid[] = "$Id: line.c,v 4.21 1999/10/20 19:10:44 mikes Exp $";
 #endif
 /*
  * Program:	Line management routines
@@ -92,8 +92,10 @@ register int    used;
     register LINE   *lp;
     register int    size;
 
-    size = (used+NBLOCK-1) & ~(NBLOCK-1);
-    if (size == 0)                          /* Assume that an empty */
+    if((size = (used+NBLOCK-1) & ~(NBLOCK-1)) > NLINE)
+      size *= 2;
+
+    if (size == 0)                    /* Assume that an empty */
       size = NBLOCK;                  /* line is for type-in. */
 
     if ((lp = (LINE *) malloc(sizeof(LINE)+(size*sizeof(CELL)))) == NULL) {
