@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: main.c,v 4.59 1999/01/20 17:27:09 mikes Exp $";
+static char rcsid[] = "$Id: main.c,v 4.63 1999/09/23 21:43:52 mikes Exp $";
 #endif
 /*
  * Program:	Main stand-alone Pine Composer routines
@@ -172,7 +172,8 @@ char    *argv[];
     update();				/* let the user know we are here */
 
 #ifdef	_WINDOWS
-    mswin_showcursor(1);	/* turn on for main window */
+    mswin_setwindow(NULL, NULL, NULL, NULL, NULL);
+    mswin_showcaret(1);			/* turn on for main window */
     mswin_allowpaste(MSWIN_PASTE_FULL);
     mswin_setclosetext("Use the ^X command to exit Pico.");
     mswin_setscrollcallback (pico_scroll_callback);
@@ -192,13 +193,26 @@ char    *argv[];
 	bp = curbp;			/* read in first file */
 	makename(bname, file_to_edit);
 	strcpy(bp->b_bname, bname);
-	strcpy(bp->b_fname, file_to_edit);
 
-	if ((gmode&MDTREE) && !in_oper_tree(file_to_edit) ||
-	    readin(file_to_edit, (viewflag==FALSE)) == ABORT) {
-	    if ((gmode&MDTREE) && !in_oper_tree(file_to_edit))
-	      emlwrite("Can't read file from outside of %s", opertree);
+	if(strlen(file_to_edit) >= NFILEN){
+	    char buf[128];
 
+	    sprintf(buf, "Filename \"%.10s...\" too long", file_to_edit);
+	    emlwrite(buf, NULL);
+	    file_to_edit = NULL;
+	}
+	else{
+	    strcpy(bp->b_fname, file_to_edit);
+	    if ((gmode&MDTREE) && !in_oper_tree(file_to_edit) ||
+		readin(file_to_edit, (viewflag==FALSE)) == ABORT) {
+		if ((gmode&MDTREE) && !in_oper_tree(file_to_edit))
+		  emlwrite("Can't read file from outside of %s", opertree);
+
+		file_to_edit = NULL;
+	    }
+	}
+
+	if(!file_to_edit){
 	    strcpy(bp->b_bname, "main");
 	    strcpy(bp->b_fname, "");
 	}
@@ -256,6 +270,7 @@ char    *argv[];
 #endif
 #ifdef	_WINDOWS
 	mswin_setdndcallback (pico_file_drop);
+	mswin_mousetrackcallback(pico_cursor);
 #endif
 	c = GetKey();
 #ifdef	MOUSE
@@ -267,6 +282,7 @@ char    *argv[];
 #endif
 #ifdef	_WINDOWS
 	mswin_cleardndcallback ();
+	mswin_mousetrackcallback(NULL);
 #endif
 
 	if(timeoutset && (c == NODATA || time_to_check())){
