@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pico.c,v 4.118 1998/12/16 03:25:57 mikes Exp $";
+static char rcsid[] = "$Id: pico.c,v 4.121 1999/06/10 22:56:11 mikes Exp $";
 #endif
 /*
  * Program:	Main Pine Composer routines
@@ -70,17 +70,11 @@ static char rcsid[] = "$Id: pico.c,v 4.118 1998/12/16 03:25:57 mikes Exp $";
 #include	"ebind.h"	/* default key bindings */
 
 
-#ifdef	ANSI
-    int  func_init(void);
-    void breplace(void *);
-    int  any_header_changes(void);
+int	func_init PROTO((void));
+void	breplace PROTO((void *));
+int	any_header_changes PROTO((void));
 #ifdef	_WINDOWS
-    int	composer_file_drop(int, int, char *);
-#endif
-#else
-    int  func_init();
-    void breplace();
-    int  any_header_changes();
+int	composer_file_drop PROTO((int, int, char *));
 #endif
 
 
@@ -163,7 +157,7 @@ PICO *pm;
       breplace(pm->msgtext);
 
 #ifdef	_WINDOWS
-    cursor_shown = mswin_showcursor(1);	/* turn on for main window */
+    cursor_shown = mswin_showcaret(1);	/* turn on for main window */
     mswin_allowpaste(MSWIN_PASTE_FULL);
     mswin_setscrollcallback (pico_scroll_callback);
 #endif
@@ -212,7 +206,7 @@ PICO *pm;
 	if(pico_all_done){
 #ifdef	_WINDOWS
 	    if(!cursor_shown)
-	      mswin_showcursor(0);
+	      mswin_showcaret(0);
 
 	    mswin_allowpaste(MSWIN_PASTE_DISABLE);
 	    mswin_setscrollcallback (NULL);
@@ -314,6 +308,7 @@ PICO *pm;
 #endif
 #ifdef	_WINDOWS
 	mswin_setdndcallback (composer_file_drop);
+	mswin_mousetrackcallback(pico_cursor);
 #endif
 	c = GetKey();
         if (term.t_nrow < 6 && c != NODATA){
@@ -331,6 +326,7 @@ PICO *pm;
 #endif
 #ifdef	_WINDOWS
 	mswin_cleardndcallback ();
+	mswin_mousetrackcallback(NULL);
 #endif
 	if(c == NODATA || time_to_check()){	/* new mail ? */
 	    if((*Pmaster->newmail)(c == NODATA ? 0 : 2, 1) >= 0){
@@ -1231,12 +1227,13 @@ mouse_get_last(f, mp)
  * register_key - register the given keystroke to accept mouse events
  */
 void
-register_key(i, rval, label, label_printer, row, col, len)
+register_key(i, rval, label, label_printer, row, col, len, kn, kl)
 int       i;
 unsigned  rval;
 char     *label;
 void      (*label_printer)();
 int       row, col, len;
+COLOR_PAIR *kn, *kl;
 {
     if(i > 11)
       return;
@@ -1252,11 +1249,37 @@ int       row, col, len;
 	free(menuitems[i].label);
 	menuitems[i].label = NULL;
     }
+    if(menuitems[i].kncp)
+        free_color_pair(&menuitems[i].kncp);
+    if(menuitems[i].klcp)
+        free_color_pair(&menuitems[i].klcp);
+    if(kn)
+      menuitems[i].kncp = new_color_pair(kn->fg, kn->bg);
+    else 
+      menuitems[i].kncp = NULL;
+    if(kl)
+      menuitems[i].klcp = new_color_pair(kl->fg, kl->bg);
+    else 
+      menuitems[i].klcp = NULL;    
 
     if(label
        && (menuitems[i].label =(char *)malloc((strlen(label)+1)*sizeof(char))))
       strcpy(menuitems[i].label, label);
 }
+
+
+int
+mouse_on_key(row, col)
+    int row, col;
+{
+    int i;
+
+    for(i = 0; i < 12; i++)
+      if(M_ACTIVE(row, col, &menuitems[i]))
+	return(TRUE);
+	
+    return(FALSE);
+}
 #endif	/* MOUSE */
 
 
@@ -1554,4 +1577,16 @@ composer_file_drop(x, y, filename)
 
     return(1);
 }
-#endif
+
+
+
+int
+pico_cursor(col, row)
+    int  col;
+    long row;
+{
+    return((row > 1 && row <= term.t_nrow - (term.t_mrow + 1))
+	     ? MSWIN_CURSOR_IBEAM
+	     : MSWIN_CURSOR_ARROW);
+}
+#endif /* _WINDOWS */
