@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: search.c,v 4.21 1998/09/11 17:13:31 mikes Exp $";
+static char rcsid[] = "$Id: search.c,v 4.24 1999/10/04 21:49:15 mikes Exp $";
 #endif
 /*
  * Program:	Searching routines
@@ -37,13 +37,17 @@ static char rcsid[] = "$Id: search.c,v 4.21 1998/09/11 17:13:31 mikes Exp $";
 
 #include	"headers.h"
 
-#ifdef	ANSI
-    int eq(int, int);
-    int expandp(char *, char *, int);
-#else
-    int eq();
-    int expandp();
-#endif
+int	eq PROTO((int, int));
+int	expandp PROTO((char *, char *, int));
+int	readnumpat PROTO((char *));
+
+
+#define	FWS_RETURN(RV)	{				\
+			    thisflag |= CFSRCH;		\
+			    curwp->w_flag |= WFMODE;	\
+			    sgarbk = TRUE;		\
+			    return(RV);			\
+			}
 
 
 /*
@@ -59,7 +63,6 @@ static char rcsid[] = "$Id: search.c,v 4.21 1998/09/11 17:13:31 mikes Exp $";
 
 
 
-
 static char *SearchHelpText[] = {
 "Help for Search Command",
 " ",
@@ -120,16 +123,17 @@ forwsearch(f, n)
       n = 1;
 
     if (n < 1)			/* search backwards */
-      return(0);
+      FWS_RETURN(0);
 
     defpat[0] = '\0';
 
     /* ask the user for the text of a pattern */
     while(1){
+
 	if (gmode & MDREPLACE)
 	  status = srpat("Search", defpat, repl_mode);
 	else
-	  status = readpattern("Search");
+	  status = readpattern("Search", TRUE);
 
 	switch(status){
 	  case TRUE:                         /* user typed something */
@@ -159,22 +163,74 @@ forwsearch(f, n)
 	  case  (CTRL|'V'):
 	    gotoeob(0, 1);
 	    mlerase();
-	    curwp->w_flag |= WFMODE;
-	    return(TRUE);
+	    FWS_RETURN(TRUE);
 
 	  case (CTRL|'Y'):
 	    gotobob(0, 1);
 	    mlerase();
-	    curwp->w_flag |= WFMODE;
-	    return(TRUE); 
+	    FWS_RETURN(TRUE); 
+
+	  case (CTRL|'T') :
+	    switch(status = readnumpat("Search to Line Number : ")){
+	      case -1 :
+		emlwrite("Search to Line Number Cancelled", NULL);
+		FWS_RETURN(FALSE);
+
+	      case  0 :
+		emlwrite("Line number must be greater than zero", NULL);
+		FWS_RETURN(FALSE);
+
+	      case -2 :
+		emlwrite("Line number must contain only digits", NULL);
+		FWS_RETURN(FALSE);
+		
+	      case -3 :
+		continue;
+
+	      default :
+		gotoline(0, status);
+		mlerase();
+		FWS_RETURN(TRUE);
+	    }
+
+	    break;
+
+	  case  (CTRL|'W'):
+	    {
+		LINE *linep = curwp->w_dotp;
+		int   offset = curwp->w_doto;
+
+		gotobop(0, 1);
+		gotobol(0, 1);
+
+		/*
+		 * if we're asked to backup and we're already
+		 *
+		 */
+		if((lastflag & CFSRCH)
+		   && linep == curwp->w_dotp
+		   && offset == curwp->w_doto
+		   && !(offset == 0 && lback(linep) == curbp->b_linep)){
+		    backchar(0, 1);
+		    gotobop(0, 1);
+		    gotobol(0, 1);
+		}
+	    }
+
+	    mlerase();
+	    FWS_RETURN(TRUE);
+
+	  case  (CTRL|'O'):
+	    if(curwp->w_dotp != curbp->b_linep){
+		gotoeop(0, 1);
+		forwchar(0, 1);
+	    }
+
+	    mlerase();
+	    FWS_RETURN(TRUE);
 
 	  case  (CTRL|'R'):        /* toggle replacement option */
-	    if (gmode & MDREPLACE){
-		if (repl_mode)
-		  repl_mode = FALSE;
-		else
-		  repl_mode = TRUE;
-	    }
+	    repl_mode = !repl_mode;
 	    break;
 
 	  default:
@@ -183,21 +239,20 @@ forwsearch(f, n)
 	    else
 	      mlerase();
 
-	    curwp->w_flag |= WFMODE;
-	    return(FALSE);
+	    FWS_RETURN(FALSE);
 	}
-	    
-	if (!(gmode & MDREPLACE)){  /* replace option is disabled */
+
+	/* replace option is disabled */
+	if (!(gmode & MDREPLACE)){
 	    strcpy(defpat, pat);
 	    break;
 	}
-	else if (search){  /* search now */ 
+	else if (search){  /* search now */
 	    strcpy(pat, defpat);  /* remember this search for the future */
 	    break;
 	}
     }
 
-
     /*
      * This code is kind of dumb.  What I want is successive C-W 's to 
      * move dot to successive occurences of the pattern.  So, if dot is
@@ -243,12 +298,11 @@ forwsearch(f, n)
 	emlwrite("", NULL);
     }
 
-    curwp->w_flag |= WFMODE;
-    sgarbk = TRUE;
-    return(status);
+    FWS_RETURN(status);
 }
 
 
+
 /* Replace a pattern with the pattern the user types in one or more times. */
 int replace_pat(defpat, wrapt)
 char *defpat;
@@ -433,28 +487,46 @@ char *repl;
 srpat(prompt, defpat, repl_mode)
 char *prompt;
 char *defpat;
-int repl_mode;
+int   repl_mode;
 {
 	register int s;
+	int	     i = 0;
 	char	     tpat[NPAT+20];
-	EXTRAKEYS    menu_pat[4];
+	EXTRAKEYS    menu_pat[7];
 
-	menu_pat[0].name  = "^Y";
-	menu_pat[0].label = "FirstLine";
-	menu_pat[0].key   = (CTRL|'Y');
-	KS_OSDATASET(&menu_pat[0], KS_NONE);
-	menu_pat[1].name  = "^V";
-	menu_pat[1].label = "LastLine";
-	menu_pat[1].key   = (CTRL|'V');
-	KS_OSDATASET(&menu_pat[1], KS_NONE);
-	menu_pat[2].name  = "^R";
-	if (repl_mode)
-	  menu_pat[2].label = "No Replace";
-	else
-	  menu_pat[2].label = "Replace";
-	menu_pat[2].key   = (CTRL|'R');
-	KS_OSDATASET(&menu_pat[2], KS_NONE);
-	menu_pat[3].name  = NULL;
+	menu_pat[i = 0].name = "^Y";
+	menu_pat[i].label    = "FirstLine";
+	menu_pat[i].key	     = (CTRL|'Y');
+	KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	menu_pat[++i].name = "^V";
+	menu_pat[i].label  = "LastLine";
+	menu_pat[i].key	   = (CTRL|'V');
+	KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	menu_pat[++i].name = "^R";
+	menu_pat[i].label  = repl_mode ? "No Replace" : "Replace";
+	menu_pat[i].key	   = (CTRL|'R');
+	KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	if(!repl_mode){
+	    menu_pat[++i].name = "^T";
+	    menu_pat[i].label  = "LineNumber";
+	    menu_pat[i].key    = (CTRL|'T');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^W";
+	    menu_pat[i].label  = "Start of Para";
+	    menu_pat[i].key    = (CTRL|'W');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^O";
+	    menu_pat[i].label  = "End of Para";
+	    menu_pat[i].key    = (CTRL|'O');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+	}
+
+	menu_pat[++i].name = NULL;
 
 	strcpy(tpat, prompt);	/* copy prompt to output string */
         if (repl_mode){
@@ -496,22 +568,87 @@ int repl_mode;
  * the search string.
  */
 
-readpattern(prompt)
+readnumpat(prompt)
 char *prompt;
 {
-	register int s;
-	char	     tpat[NPAT+20];
-	EXTRAKEYS    menu_pat[3];
+    register int s;
+    int		 i, n;
+    char	 tpat[NPAT+20];
+    EXTRAKEYS    menu_pat[2];
 
-	menu_pat[0].name  = "^Y";
-	menu_pat[0].label = "FirstLine";
-	menu_pat[0].key   = (CTRL|'Y');
-	KS_OSDATASET(&menu_pat[0], KS_NONE);
-	menu_pat[1].name  = "^V";
-	menu_pat[1].label = "LastLine";
-	menu_pat[1].key   = (CTRL|'V');
-	KS_OSDATASET(&menu_pat[1], KS_NONE);
-	menu_pat[2].name  = NULL;
+    menu_pat[i = 0].name  = "^T";
+    menu_pat[i].label	  = "No Line Number";
+    menu_pat[i].key	  = (CTRL|'T');
+    KS_OSDATASET(&menu_pat[i++], KS_NONE);
+
+    menu_pat[i].name  = NULL;
+
+    tpat[0] = '\0';
+    while(1)
+      switch(mlreplyd(prompt, tpat, NPAT, QNORML, menu_pat)){
+	case TRUE :
+	  if(*tpat){
+	      for(i = n = 0; tpat[i]; i++)
+		if(strchr("0123456789", tpat[i])){
+		    n = (n * 10) + (tpat[i] - '0');
+		}
+		else
+		  return(-2);
+
+	      return(n);
+	  }
+
+	case FALSE :
+	default :
+	  return(-1);
+
+	case (CTRL|'T') :
+	  return(-3);
+
+	case (CTRL|'L') :
+	case HELPCH :
+	  break;
+      }
+}	    
+
+
+readpattern(prompt, text_mode)
+char *prompt;
+int   text_mode;
+{
+	register int s;
+	int	     i;
+	char	     tpat[NPAT+20];
+	EXTRAKEYS    menu_pat[6];
+
+	menu_pat[i = 0].name = "^Y";
+	menu_pat[i].label    = "FirstLine";
+	menu_pat[i].key	     = (CTRL|'Y');
+	KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	menu_pat[++i].name = "^V";
+	menu_pat[i].label  = "LastLine";
+	menu_pat[i].key	   = (CTRL|'V');
+	KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	if(text_mode){
+	    menu_pat[++i].name = "^T";
+	    menu_pat[i].label  = "LineNumber";
+	    menu_pat[i].key    = (CTRL|'T');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^W";
+	    menu_pat[i].label  = "Start of Para";
+	    menu_pat[i].key    = (CTRL|'W');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^O";
+	    menu_pat[i].label  = "End of Para";
+	    menu_pat[i].key    = (CTRL|'O');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+	}
+
+	menu_pat[++i].name = NULL;
 
 	strcpy(tpat, prompt);	/* copy prompt to output string */
         if(pat[0] != '\0'){
