@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: word.c,v 4.30 1998/04/02 20:48:47 mikes Exp $";
+static char rcsid[] = "$Id: word.c,v 4.31 1999/06/08 21:36:24 jfrankli Exp $";
 #endif
 /*
  * Program:	Word at a time routines
@@ -49,20 +49,38 @@ wrapword()
     register int bp;			/* index to wrap on */
     register int first = -1;
     register int i;
+    int wasperiod = 0;                  /* true if previous char was .?! etc. */
 
     if(curwp->w_doto <= 0)		/* no line to wrap? */
       return(FALSE);
 
     for(bp = cnt = i = 0; cnt < llength(curwp->w_dotp) && !bp; cnt++, i++){
 	if(isspace((unsigned char) lgetc(curwp->w_dotp, cnt).c)){
+	  if(!wasperiod){
 	    first = 0;
 	    if(lgetc(curwp->w_dotp, cnt).c == TAB)
 	      while(i+1 & 0x07)
 		i++;
+	  }
+	}
+	else{
+	  if(!first)
+	    first = cnt;
+	  switch(lgetc(curwp->w_dotp,cnt).c){
+	    case '.':
+	    case '?':
+	    case '!':
+	    case ':':
+	    case ';':
+	    case '\"':
+	    case ')':
+	      wasperiod = 1;
+	      break;
+	    default:
+	      wasperiod = 0;
+	      break;
+	  }
 	}
-	else if(!first)
-	  first = cnt;
-
 	if(first > 0 && i >= fillcol)
 	  bp = first;
     }
