@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: adrbklib.h,v 4.128 1999/01/21 17:35:12 hubert Exp $
+  $Id: adrbklib.h,v 4.132 1999/09/09 21:41:42 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -348,10 +348,9 @@ typedef enum {Local, Imap} AdrbkType;
 #define OUT_OF_SORTS	0x100	/* sort order is messed up                 */
 #define FIRST_CHANGE	0x200	/* first change this session               */
 
-#define IMAP_VALID_CHK_INTERVAL	( 5L * 60L)
-#define IMAP_IDLE_TIMEOUT	(10L * 60L)
-#define FILE_VALID_CHK_INTERVAL (      15L)
-#define LOW_FREQ_CHK_INTERVAL	(30L * 60L)
+#define IMAP_IDLE_TIMEOUT	(10L * 60L)	/* seconds */
+#define FILE_VALID_CHK_INTERVAL (      15L)	/* seconds */
+#define LOW_FREQ_CHK_INTERVAL	(30L)		/* minutes */
 
 typedef struct adrbk {
     AdrbkType	  type;                /* type of address book               */
@@ -830,7 +829,8 @@ typedef enum {AddrBookScreen,	   /* maintenance screen                     */
 	      SelectNickTake,	   /* Same as SelectNick but different help  */
 	      SelectNickCom,	   /* Same as SelectNick but from composer   */
 	      SelectManyNicks,	   /* Returns list of nicks */
-	      SelectAddr	   /* Returns single address */
+	      SelectAddr,	   /* Returns single address */
+	      SelectAddrNoFull	   /* Returns single address without fullname */
 	     } AddrBookArg;
 
 typedef struct save_state_struct {
@@ -1108,7 +1108,7 @@ typedef struct _cust_filt {
  * Prototypes
  */
 int            ab_add_abook PROTO((int, int));
-int            ab_compose_to_addr PROTO((long, int));
+int            ab_compose_to_addr PROTO((long, int, int));
 int            ab_save PROTO((struct pine *, AdrBk *, long, int, int));
 int            ab_del_abook PROTO((long, int, char **));
 int            ab_agg_delete PROTO((struct pine *, int));
@@ -1119,9 +1119,9 @@ int            ab_shuffle PROTO((PerAddrBook *, int *, int, char **));
 char          *abe_to_nick_or_addr_string PROTO((AdrBk_Entry *,
 						 AddrScrn_Disp *, int));
 char          *addr_book_change_list PROTO((char **));
-int            adrbk_check_and_fix PROTO((PerAddrBook *, int, int));
-int            adrbk_check_and_fix_all PROTO((int, int));
-void           adrbk_check_validity PROTO((AdrBk *, int));
+int            adrbk_check_and_fix PROTO((PerAddrBook *, int, int, int));
+int            adrbk_check_and_fix_all PROTO((int, int, int));
+void           adrbk_check_validity PROTO((AdrBk *, long));
 int            adrbk_check_all_validity_now PROTO((void));
 char          *addr_book_nick_for_edit PROTO((char **));
 void           adrbk_update_metafile PROTO((char *, char *, unsigned long,
