@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.83 1999/09/07 17:11:25 hubert Exp $";
+static char rcsid[] = "$Id: args.c,v 4.84 1999/11/08 23:00:10 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -133,7 +133,7 @@ char *args_pine_args[] = {
 " -nr\t\tSpecial mode for UWIN",
 " -a\t\tSpecial anonymous mode for UWIN",
 #if	defined(_WINDOWS)
-" -registry <cmd>\tcWhere cmd is set,clear,dump",
+" -registry <cmd>\tWhere cmd is set,clear,dump",
 #endif
 " -<option>=<value>   Assign <value> to the pinerc option <option>",
 "\t\t     e.g. -signature-file=sig1",
