@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c,v 4.111 1999/01/06 19:50:30 mikes Exp $";
+static char rcsid[] = "$Id: help.c,v 4.115 1999/09/14 20:21:50 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -181,6 +181,7 @@ helper_internal(text, frag, title, flags)
 {
     char	  **shown_text;
     int		    cmd = MC_NONE;
+    long	    offset = 0L;
     char	   *error = NULL, tmp_title[MAX_SCREEN_COLS + 1];
     STORE_S	   *store;
     HANDLE_S	   *handles = NULL, *htmp;
@@ -263,7 +264,12 @@ helper_internal(text, frag, title, flags)
 		sargs.text.text	   = so_text(store);
 		sargs.text.src	   = CharStar;
 		sargs.text.desc	   = "help text";
-		sargs.text.handles = handles;
+		if(sargs.text.handles = handles)
+		  while(sargs.text.handles->type == URL
+			&& !sargs.text.handles->h.url.path
+			&& sargs.text.handles->next)
+		    sargs.text.handles = sargs.text.handles->next;
+
 		if(!(sargs.bar.title = title)){
 		    if(!struncmp(shown_text[0], "<html>", 6)){
 			char *p;
@@ -357,9 +363,16 @@ helper_internal(text, frag, title, flags)
 		  clrbitn(HLP_ALL_KEY, sargs.keys.bitmap);
 
 		if(frag){
-		    sargs.start.on   = Fragment;
-		    sargs.start.frag = frag;
+		    sargs.start.on	 = Fragment;
+		    sargs.start.loc.frag = frag;
+		    frag		 = NULL; /* ignore next time */
 		}
+		else if(offset){
+		    sargs.start.on	   = Offset;
+		    sargs.start.loc.offset = offset;
+		}
+		else
+		  sargs.start.on = FirstPage;
 
 #ifdef	_WINDOWS
 		sargs.mouse.popup = (flags & HLPD_FROMHELP)
@@ -368,6 +381,8 @@ helper_internal(text, frag, title, flags)
 
 		cmd = scrolltool(&sargs);
 
+		offset = sargs.start.loc.offset;
+
 		if(F_ON(F_BLANK_KEYMENU,ps_global))
 		  FOOTER_ROWS(ps_global) = 1;
 
@@ -403,7 +418,7 @@ helper(text, title, flags)
 
 
 
-int
+void
 init_helper_getc(help_txt)
     char **help_txt;
 {
