@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.146 1999/10/11 22:11:42 mikes Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.148 1999/11/04 01:27:56 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -63,6 +63,7 @@ static char rcsid[] = "$Id: imap.c,v 4.146 1999/10/11 22:11:42 mikes Exp $";
 typedef struct _mmlogin_s {
     char	   *user,
 		   *passwd;
+    unsigned	    altflag:1;
     STRLIST_S	   *hosts;
     struct _mmlogin_s *next;
 } MMLOGIN_S;
@@ -77,13 +78,14 @@ typedef	struct _se_app_s {
 /*
  * Internal prototypes
  */
+void  mm_login_alt_cue PROTO((NETMBX *));
 long  imap_seq_exec PROTO((MAILSTREAM *, char *,
 			   long (*) PROTO((MAILSTREAM *, long, void *)),
 			   void *));
 long  imap_seq_exec_append PROTO((MAILSTREAM *, long, void *));
-char *imap_get_user PROTO((MMLOGIN_S *, STRLIST_S *));
-int   imap_get_passwd PROTO((MMLOGIN_S *, char *, char *, STRLIST_S *));
-void  imap_set_passwd PROTO((MMLOGIN_S **, char *, char *, STRLIST_S *));
+char *imap_get_user PROTO((MMLOGIN_S *, STRLIST_S *, int));
+int   imap_get_passwd PROTO((MMLOGIN_S *, char *, char *, STRLIST_S *, int));
+void  imap_set_passwd PROTO((MMLOGIN_S **, char *, char *, STRLIST_S *, int));
 int   imap_same_host PROTO((STRLIST_S *, STRLIST_S *));
 #ifdef	PASSFILE
 char  xlate_in PROTO((int));
@@ -91,8 +93,8 @@ char  xlate_out PROTO((char));
 char *passfile_name PROTO((char *, char *));
 int   read_passfile PROTO((char *, MMLOGIN_S **));
 void  write_passfile PROTO((char *, MMLOGIN_S *));
-int   get_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *));
-void  set_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *));
+int   get_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
+void  set_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
 #endif
 
 
@@ -575,6 +577,9 @@ mm_login(mb, user, pwd, trial)
         return;
     }
 
+    /* make sure errors are seen */
+    flush_status_messages(0);
+
     /*
      * set up host list for sybil servers...
      */
@@ -596,13 +601,15 @@ mm_login(mb, user, pwd, trial)
 	strcpy(user, (*mb->user) ? mb->user : ps_global->VAR_USER_ID);
 
 	/* try last working password associated with this host. */
-	if(imap_get_passwd(mm_login_list, pwd, user, hostlist))
+	if(imap_get_passwd(mm_login_list, pwd, user, hostlist, mb->altflag))
 	  return;
 
 #ifdef	PASSFILE
 	/* check to see if there's a password left over from last session */
-	if(get_passfile_passwd(ps_global->pinerc, pwd, user, &hostlist[0])){
-	    imap_set_passwd(&mm_login_list, pwd, user, &hostlist[0]);
+	if(get_passfile_passwd(ps_global->pinerc, pwd,
+			       user, &hostlist[0], mb->altflag)){
+	    imap_set_passwd(&mm_login_list, pwd, user,
+			    &hostlist[0], mb->altflag);
 	    return;
 	}
 #endif
@@ -612,17 +619,11 @@ mm_login(mb, user, pwd, trial)
 	 * with our local user name, see if we've visited this
 	 * host before as someone else...
 	 */
-	if(!*mb->user && (last = imap_get_user(mm_login_list, hostlist)))
+	if(!*mb->user
+	   && (last = imap_get_user(mm_login_list, hostlist, mb->altflag)))
 	  strcpy(user, last);
     }
 
-    if(mb->altflag){
-	MoveCursor(0, ps_global->ttyo->screen_cols - 1);
-	StartInverse();
-	Write_to_screen("+");
-	EndInverse();
-    }
-
     ps_global->mangled_footer = 1;
     if(!*mb->user){
 	help = NO_HELP;
@@ -654,6 +655,8 @@ mm_login(mb, user, pwd, trial)
 	strcpy(&prompt[i], last);
 
 	while(1) {
+	    mm_login_alt_cue(mb);
+
 	    flags = OE_APPEND_CURRENT;
 	    rc = optionally_enter(user, q_line, 0, MAILTMPLEN - 1,
 				  prompt, NULL, help, &flags);
@@ -673,16 +676,8 @@ mm_login(mb, user, pwd, trial)
     else
       strcpy(user, mb->user);
 
-    if(!user[0]){
-	if(mb->altflag){
-	    MoveCursor(0, ps_global->ttyo->screen_cols - 1);
-	    StartInverse();
-	    Write_to_screen(" ");
-	    EndInverse();
-	}
-
-	return;
-    }
+    if(!user[0])
+      return;
 
     help = NO_HELP;
 
@@ -733,24 +728,20 @@ mm_login(mb, user, pwd, trial)
 
     *pwd = '\0';
     while(1) {
+	mm_login_alt_cue(mb);
+
 	flags = OE_PASSWD;
         rc = optionally_enter(pwd, q_line, 0, MAILTMPLEN - 1,
 			      prompt, NULL, help, &flags);
         if(rc == 3) {
             help = help == NO_HELP ? h_oe_passwd : NO_HELP;
-            continue;
         }
-        if(rc != 4)
+	else if(rc == 4){
+	}
+	else
           break;
     }
 
-    if(mb->altflag){
-	MoveCursor(0, ps_global->ttyo->screen_cols - 1);
-	StartInverse();
-	Write_to_screen(" ");
-	EndInverse();
-    }
-
     if(rc == 1 || !pwd[0]) {
 	if(!*mb->user){
 	    strcpy(user, junk);
@@ -763,15 +754,41 @@ mm_login(mb, user, pwd, trial)
     }
 
     /* remember the password for next time */
-    imap_set_passwd(&mm_login_list, pwd, user, hostlist);
+    imap_set_passwd(&mm_login_list, pwd, user, hostlist, mb->altflag);
 #ifdef	PASSFILE
     /* if requested, remember it on disk for next session */
-    set_passfile_passwd(ps_global->pinerc, pwd, user, hostlist);
+    set_passfile_passwd(ps_global->pinerc, pwd, user, hostlist, mb->altflag);
 #endif
 }
 
 
 
+void
+mm_login_alt_cue(mb)
+    NETMBX *mb;
+{
+    if(mb->altflag){
+	COLOR_PAIR  *lastc;
+
+	lastc = pico_set_colors(ps_global->VAR_TITLE_FORE_COLOR,
+				ps_global->VAR_TITLE_BACK_COLOR,
+				PSC_REV | PSC_RET);
+
+	mark_titlebar_dirty();
+	PutLine0(0, ps_global->ttyo->screen_cols - 1, "+");
+
+	if(lastc){
+	    (void)pico_set_colorp(lastc, PSC_NONE);
+	    free_color_pair(&lastc);
+	}
+
+	fflush(stdout);
+    }
+}
+
+
+
+
 /*----------------------------------------------------------------------
        Receive notification of an error writing to disk
       
@@ -1478,9 +1495,10 @@ imap_same_host(hl1, hl2)
 
 
 char *
-imap_get_user(m_list, hostlist)
+imap_get_user(m_list, hostlist, altflag)
     MMLOGIN_S *m_list;
     STRLIST_S *hostlist;
+    int	       altflag;
 {
     MMLOGIN_S *l;
     STRLIST_S *h_list;
@@ -1488,7 +1506,7 @@ imap_get_user(m_list, hostlist)
     
     for(l = m_list; l; l = l->next)
       /* host name and user name must match */
-      if(imap_same_host(l->hosts, hostlist))
+      if(imap_same_host(l->hosts, hostlist) && altflag == m_list->altflag)
 	return(l->user);
 
     return(NULL);
@@ -1497,10 +1515,11 @@ imap_get_user(m_list, hostlist)
 
 
 int
-imap_get_passwd(m_list, passwd, user, hostlist)
+imap_get_passwd(m_list, passwd, user, hostlist, altflag)
     MMLOGIN_S *m_list;
     char      *passwd, *user;
     STRLIST_S *hostlist;
+    int	       altflag;
 {
     MMLOGIN_S *l;
     STRLIST_S *h_list;
@@ -1509,7 +1528,8 @@ imap_get_passwd(m_list, passwd, user, hostlist)
     for(l = m_list; l; l = l->next)
       /* host name and user name must match */
       if(imap_same_host(l->hosts, hostlist)
-	 && (!*user || !strcmp(user, l->user))){
+	 && (!*user || !strcmp(user, l->user))
+	 && (*user && l->altflag == altflag)){
 	  strcpy(user, l->user);
 	  strcpy(passwd, l->passwd);
 	  return(TRUE);
@@ -1521,7 +1541,7 @@ imap_get_passwd(m_list, passwd, user, hostlist)
 
 
 void
-imap_set_passwd(l, passwd, user, hostlist)
+imap_set_passwd(l, passwd, user, hostlist, altflag)
     MMLOGIN_S **l;
     char       *passwd, *user;
     STRLIST_S  *hostlist;
@@ -1530,7 +1550,8 @@ imap_set_passwd(l, passwd, user, hostlist)
 
     for(; *l; l = &(*l)->next)
       if(imap_same_host((*l)->hosts, hostlist)
-	 && !strcmp(user, (*l)->user))
+	 && !strcmp(user, (*l)->user)
+	 && altflag == (*l)->altflag)
 	if(strcmp(passwd, (*l)->passwd)){
 	    fs_give((void **)&(*l)->passwd);
 	    break;
@@ -1548,6 +1569,8 @@ imap_set_passwd(l, passwd, user, hostlist)
 
     (*l)->passwd = cpystr(passwd);
 
+    (*l)->altflag = altflag;
+
     if(!(*l)->user)
       (*l)->user = cpystr(user);
 
@@ -1695,7 +1718,7 @@ read_passfile(pinerc, l)
     char       *pinerc;
     MMLOGIN_S **l;
 {
-    char  tmp[MAILTMPLEN], *ui[3];
+    char  tmp[MAILTMPLEN], *ui[4];
     int   i, j, n;
     FILE *fp;
 
@@ -1712,8 +1735,8 @@ read_passfile(pinerc, l)
 	if(i && tmp[i-1] == '\n')
 	  tmp[i-1] = '\0';			/* blast '\n' */
 
-	ui[0] = ui[1] = ui[2] = NULL;
-	for(i = 0, j = 0; tmp[i] && j < 3; j++){
+	ui[0] = ui[1] = ui[2] = ui[3] = NULL;
+	for(i = 0, j = 0; tmp[i] && j < 4; j++){
 	    for(ui[j] = &tmp[i]; tmp[i] && tmp[i] != '\t'; i++)
 	      ;					/* find end of data */
 
@@ -1723,10 +1746,11 @@ read_passfile(pinerc, l)
 
 	if(ui[0] && ui[1] && ui[2]){		/* valid field? */
 	    STRLIST_S hostlist[1];
+	    int	      flags = ui[3] ? atoi(ui[3]) : 0;
 
 	    hostlist[0].name = ui[2];
 	    hostlist[0].next = NULL;
-	    imap_set_passwd(l, ui[0], ui[1], hostlist);
+	    imap_set_passwd(l, ui[0], ui[1], hostlist, flags & 0x01);
 	}
     }
 
@@ -1751,7 +1775,8 @@ write_passfile(pinerc, l)
 
     for(n = 0; l; l = l->next, n++){
 	/*** do any necessary ENcryption here ***/
-	sprintf(tmp, "%s\t%s\t%s\n", l->passwd, l->user, l->hosts->name);
+	sprintf(tmp, "%s\t%s\t%s\t%d\n", l->passwd, l->user,
+		l->hosts->name, l->altflag);
 	xlate_key = n;
 	for(i = 0; tmp[i]; i++)
 	  tmp[i] = xlate_in(tmp[i]);
@@ -1770,12 +1795,15 @@ write_passfile(pinerc, l)
  *            as the pinerc with the name defined above.
  */
 int
-get_passfile_passwd(pinerc, passwd, user, hostlist)
+get_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
     char      *pinerc, *passwd, *user;
     STRLIST_S *hostlist;
+    int	       altflag;
 {
     return((passfile_cache || read_passfile(pinerc, &passfile_cache))
-	     ? imap_get_passwd(passfile_cache, passwd, user, hostlist) : 0);
+	     ? imap_get_passwd(passfile_cache, passwd,
+			       user, hostlist, altflag)
+	     : 0);
 }
 
 
@@ -1786,14 +1814,15 @@ get_passfile_passwd(pinerc, passwd, user, hostlist)
  *            as the pinerc with the name defined above.
  */
 void
-set_passfile_passwd(pinerc, passwd, user, hostlist)
+set_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
     char      *pinerc, *passwd, *user;
     STRLIST_S *hostlist;
+    int	       altflag;
 {
     if((passfile_cache || read_passfile(pinerc, &passfile_cache))
        && want_to("Preserve password on DISK for next login", 'y', 'x',
 		  NO_HELP, WT_NORM) == 'y'){
-	imap_set_passwd(&passfile_cache, passwd, user, hostlist);
+	imap_set_passwd(&passfile_cache, passwd, user, hostlist, altflag);
 	write_passfile(pinerc, passfile_cache);
     }
 }
