@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailcap.c,v 1.71 1998/11/18 18:00:12 mikes Exp $";
+static char rcsid[] = "$Id: mailcap.c,v 1.73 1999/04/21 00:13:22 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -738,12 +738,15 @@ PARAMETER    *params;
 
     dprint(5, (debugfile, "- mc_passes_test -\n"));
 
-    if(mc->testcommand && *mc->testcommand)
-	cmd = mc_bld_test_cmd(mc->testcommand, type, subtype, params);
+    if(mc->testcommand
+       && *mc->testcommand
+       && !(cmd = mc_bld_test_cmd(mc->testcommand, type, subtype, params)))
+      return(FALSE);	/* couldn't be built */
     
     if(!mc->testcommand || !cmd || !*cmd){
 	if(cmd)
 	  fs_give((void **)&cmd);
+
 	dprint(7, (debugfile, "no test command, so Pass\n"));
 	return 1;
     }
@@ -794,7 +797,8 @@ mailcap_build_command(type, subtype, params, tmp_file, needsterm)
     if(needsterm)
       *needsterm = mc->needsterminal;
 
-    command = mc_cmd_bldr(mc->command, type, subtype, params, tmp_file);
+    if(!(command = mc_cmd_bldr(mc->command, type, subtype, params, tmp_file)))
+      command = cpystr("");
 
     dprint(5, (debugfile, "built command: %s\n", command));
 
@@ -871,6 +875,12 @@ mc_cmd_bldr(controlstring, type, subtype, parameter, tmp_file)
 		break;
 
 	      case '{':			/* insert requested MIME param */
+		if(F_OFF(F_DO_MAILCAP_PARAM_SUBST, ps_global)){
+		    dprint(2,
+			   (debugfile, "mc_cmd_bldr: param subs %s\n", from));
+		   return(NULL);
+		}
+
 		s = strindex(from, '}');
 		if(!s){
 		    q_status_message1(SM_ORDER, 0, 4,
@@ -956,7 +966,7 @@ mc_cmd_bldr(controlstring, type, subtype, parameter, tmp_file)
       sprintf(to, MC_ADD_TMP, tmp_file);
 
     return(cpystr(tmp_20k_buf));
-} 
+}
 
 
 /*
@@ -1098,32 +1108,33 @@ mt_srch_mime_type(mt_operator, mt_map)
     if(pathcopy)
       fs_give((void **)&pathcopy);
 
-    if(!rv)
 #ifdef	_WINDOWS
-      if(mt_operator == mt_srch_by_ext){
-	  char buf[256];
+    if(!rv){
+	if(mt_operator == mt_srch_by_ext){
+	    char buf[256];
 
-	  if(mswin_reg_mime_type(mt_map->from.ext, buf)){
-	      if(s = strindex(buf, '/')){
-		  *s++ = '\0';
-		  mt_map->to.mime.type = mt_translate_type(buf);
-		  mt_map->to.mime.subtype = cpystr(s);
-		  rv = 1;
-	      }
-	  }
-      }
-      else if(mt_operator == mt_srch_by_type){
-	  if(mswin_reg_mime_ext(mt_map->from.mime_type,
-				mt_map->to.ext)){
-	      if(*(s = mt_map->to.ext) == '.')
-		while(*s = *(s+1))
-		  s++;
+	    if(mswin_reg_mime_type(mt_map->from.ext, buf)){
+		if(s = strindex(buf, '/')){
+		    *s++ = '\0';
+		    mt_map->to.mime.type = mt_translate_type(buf);
+		    mt_map->to.mime.subtype = cpystr(s);
+		    rv = 1;
+		}
+	    }
+	}
+	else if(mt_operator == mt_srch_by_type){
+	    if(mswin_reg_mime_ext(mt_map->from.mime_type,
+				  mt_map->to.ext)){
+		if(*(s = mt_map->to.ext) == '.')
+		  while(*s = *(s+1))
+		    s++;
 
-	      rv = 1;
-	  }
-      }
-      else
-	panic("Unhandled mime type search");
+		rv = 1;
+	    }
+	}
+	else
+	  panic("Unhandled mime type search");
+    }
 #endif
 
 
