@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.102 1999/10/08 16:46:32 mikes Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.103 1999/11/02 00:12:32 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -207,11 +207,13 @@ new_mail(force, time_for_check_point, flags)
 		    pine_state->new_mail_count);
 
 	n = any_lflagged(pine_state->msgmap, MN_EXLD);
-	process_filter_patterns(pine_state->mail_stream,
-				pine_state->msgmap,
-				ps_global->new_mail_count);
-	if((n -= any_lflagged(pine_state->msgmap, MN_EXLD)) < 0)
-	  pine_state->new_mail_count += n;
+	if(pine_state->new_mail_count){
+	    process_filter_patterns(pine_state->mail_stream,
+				    pine_state->msgmap,
+				    ps_global->new_mail_count);
+	    if((n -= any_lflagged(pine_state->msgmap, MN_EXLD)) < 0)
+	      pine_state->new_mail_count += n;
+	}
 
 	/*
 	 * Lastly, worry about sorting if we got something new, otherwise
@@ -258,11 +260,13 @@ new_mail(force, time_for_check_point, flags)
 		    pine_state->inbox_new_mail_count);
 
 	n = any_lflagged(pine_state->inbox_msgmap, MN_EXLD);
-	process_filter_patterns(pine_state->inbox_stream,
-				pine_state->inbox_msgmap,
-				pine_state->inbox_new_mail_count);
-	if((n -= any_lflagged(pine_state->inbox_msgmap, MN_EXLD)) < 0)
-	  pine_state->inbox_new_mail_count += n;
+	if(pine_state->inbox_new_mail_count){
+	    process_filter_patterns(pine_state->inbox_stream,
+				    pine_state->inbox_msgmap,
+				    pine_state->inbox_new_mail_count);
+	    if((n -= any_lflagged(pine_state->inbox_msgmap, MN_EXLD)) < 0)
+	      pine_state->inbox_new_mail_count += n;
+	}
 
         if(pine_state->inbox_new_mail_count > 0) {
             inbox_mail_since_command       += pine_state->inbox_new_mail_count;
