@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-bsd.h,v 5.44 1996/06/05 17:12:38 mikes Exp $
+  $Id: os-bsd.h,v 5.53 1998/10/07 21:51:21 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -20,7 +20,7 @@
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1996 by the University of Washington.
+   1989-1998 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -41,6 +41,23 @@
 #define _OS_INCLUDED
 
 
+char *strstr();
+unsigned long strtoul();
+void *malloc();
+void free();
+/*
+ * This BSD stuff is lame. We'd like to just include ../c-client/osdep.h
+ * like we do for the other systems, but that osdep.h includes sys/dir.h which
+ * has already been included by ../pico/headers.h, and the double include
+ * chokes us. So these are the other local includes from ../c-client/osdep.h.
+ */
+#include "../c-client/env_unix.h"
+#include "../c-client/fs.h"
+#include "../c-client/ftl.h"
+#include "../c-client/nl.h"
+#include "../c-client/tcp.h"
+
+
 /*----------------------------------------------------------------------
 
    This first section has some constants that you may want to change
@@ -58,17 +75,23 @@
    there should be a dummy disk_quota() routine in os-xxx.c so that defining
    this won't be harmful anyway.
  ----*/
-/* #define USE_QUOTAS  /* comment out if you never want quotas checked */
+/* #define USE_QUOTAS */
 
 
 
 /*----------------------------------------------------------------------
-   Define this if you want to allow the users to change their From header
-   line when they send out mail.  The users will still have to configure
-   either default-composer-hdrs or customized-hdrs to get at the From
-   header, even if this is set.
+   Define this if you want to be sure to not allow users to change their
+   From header line when they send out mail. Even if you don't define
+   this the default is to not allow From to be changed. The user would have
+   to edit (by hand) "allow-changing-from" into their .pinerc feature list
+   or the sys-admin would have to include it in the default feature-list.
+   Even after that, the user still has to include it in either their
+   default-composer-hdrs or customized-hdrs to get at it. Instead of
+   defining NEVER_ALLOW_CHANGING_FROM, an easier way of preventing From
+   changing is to put the feature "no-allow-changing-from" in the
+   /usr/local/lib/pine.conf.fixed file.
  ----*/
-/* #define ALLOW_CHANGING_FROM  /* comment out to not allow changing From */
+/* #define NEVER_ALLOW_CHANGING_FROM */
 
 
 
@@ -79,7 +102,7 @@
    NOTE: You'll also have to make sure the appropriate osdep/postreap.*
          file is included in the os-*.ic file for your system.
  ----*/
-/* #define BACKGROUND_POST  /* comment out to disable posting from child */
+/* #define BACKGROUND_POST */
 
 
 
@@ -178,6 +201,7 @@
 #define DF_SAVE_BY_SENDER         "no"
 #define DF_SORT_KEY               "arrival"
 #define DF_AB_SORT_RULE           "fullname-with-lists-last"
+#define DF_FLD_SORT_RULE          "alphabetical"
 #define DF_SAVED_MSG_NAME_RULE    "default-folder"
 #define DF_FCC_RULE               "default-fcc"
 #define DF_STANDARD_PRINTER       "lpr"
@@ -192,6 +216,7 @@
 #define DF_LOCAL_FULLNAME         "Local Support"
 #define DF_LOCAL_ADDRESS          "postmaster"
 #define DF_KBLOCK_PASSWD_COUNT    "1"
+#define DF_REMOTE_ABOOK_HISTORY   "3"
 
 /*----------------------------------------------------------------------
    The default printer when pine starts up for the first time with no printer
@@ -200,148 +225,6 @@
 
 
 
-/*----------------------------------------------------------------------
-
-   OS dependencies, BSD 4.3 version.  See also the os-bsd.c files.
-   The following stuff may need to be changed for a new port, but once
-   the port is done, it won't change.  At the bottom of the file are a few
-   constants that you may want to configure differently than they
-   are configured, but probably not.
-
- ----*/
-
-
-
-/*----------------- Are we ANSI? ---------------------------------------*/
-/* #define ANSI          /* this is an ANSI compiler */
-
-/*------ If our compiler doesn't understand type void ------------------*/
-/* #define void char     /* no void in compiler */
-
-/*-------- Standard ANSI functions usually defined in stdlib.h ---------*/
-/* #include <stdlib.h>  /* declarations of many common functions */
-/*---- Declare getenv() if not already declared ------------------------*/
-char *getenv();      /* often declared in stdlib.h */
-FILE *tmpfile();
-extern int errno;
-/* Provide these two functions in os.c */
-typedef long fpos_t;
-int fgetpos(); int fsetpos();
-
-
-/******* These are included in c-client/osdep.h. ************************/
-/*------- Standard string functions ------------------------------------*/
-#include <string.h>
-/*---- Declare these if not already declared ---------------------------*/
-/* char *strcpy(), *strncpy();  /* usually in string.h */
-
-
-/*------- Include types.h if you have it -------------------------------*/
-#include <sys/types.h>
-
-
-/*------- Some more includes that should usually be correct ------------*/
-#include <pwd.h>
-#include <sys/wait.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <netdb.h>
-
-
-
-/*----------------- locale.h -------------------------------------------*/
-/* #include <locale.h>  /* To make matching and sorting work right */
-
-
-
-/*----------------- time.h ---------------------------------------------*/
-/* #include <time.h> */
-/* plain time.h isn't enough on some systems */
-#include <sys/time.h>  /* For struct timeval usually in time.h */ 
-
-
-
-/*--------------- signal.h ---------------------------------------------*/
-#include <signal.h>      /* sometimes both required, sometimes */
-/* #include <sys/signal.h>  /* only one or the other */
-
-/* #define SigType void     /* value returned by sig handlers is void */
-#define SigType int      /* value returned by sig handlers is int */
-
-/* #define POSIX_SIGNALS    /* use POSIX signal semantics (ttyin.c) */
-/* #define SYSV_SIGNALS    /* use System-V signal semantics (ttyin.c) */
-
-
-
-
-/*-------------- A couple typedef's for integer sizes ------------------*/
-typedef unsigned int usign32_t;
-typedef unsigned short usign16_t;
-
-
-
-/*-------------- qsort argument type -----------------------------------*/
-/* #define QSType void  /* qsort arg is of type void * */
-#define QSType char  /* qsort arg is of type char * */
-
-
-
-/*-------------- fcntl flag to set non-blocking IO ---------------------*/
-/*#define	NON_BLOCKING_IO	O_NONBLOCK		/* POSIX style */
-#define	NON_BLOCKING_IO	FNDELAY		/* good ol' bsd style  */
-
-
-
-/*------ how help text is referenced (always char ** on Unix) ----------*/
-#define HelpType char **
-#define NO_HELP (char **)NULL
-
-
-
-/*
- * Choose one of the following three terminal drivers
- */
-
-/*--------- Good 'ol BSD -----------------------------------------------*/
-#include <sgtty.h>      /* BSD-based systems */
-
-/*--------- System V terminal driver -----------------------------------*/
-/* #define HAVE_TERMIO     /* this is for pure System V */
-/* #include <termio.h>     /* Sys V */
-
-/*--------- POSIX terminal driver --------------------------------------*/
-/* #define HAVE_TERMIOS    /* this is an alternative */
-/* #include <termios.h>    /* POSIX */
-
-
-
-/*-------- Use poll system call instead of select ----------------------*/
-/* #define USE_POLL        /* use the poll() system call instead of select() */
-
-
-
-/*-------- Use terminfo database instead of termcap --------------------*/
-/* #define USE_TERMINFO    /* use terminfo instead of termcap */
-
-
-
-/*-- What argument does wait(2) take? Define this if it is a union -----*/
-#define HAVE_WAIT_UNION  /* the arg to wait is a union wait * */
-
-
-
-/*-------- Is window resizing available? -------------------------------*/
-#if defined(TIOCGWINSZ) && defined(SIGWINCH)
-#define RESIZING  /* SIGWINCH and friends */
-#endif
-
-
-
-/*-------- If no vfork, use regular fork -------------------------------*/
-/* #define vfork fork  /* vfork is just a lightweight fork, so can use fork */
-
-
-
 /*----- The usual sendmail configuration for sending mail on Unix ------*/
 #define SENDMAIL	"/usr/lib/sendmail"
 #define SENDMAILFLAGS	"-bs -odb -oem"	/* send via smtp with backgroud
@@ -351,7 +234,7 @@ typedef unsigned short usign16_t;
 /*----------------------------------------------------------------------
    If no nntp-servers are defined, this program will be used to post news.
  ----*/
-/* #define SENDNEWS	"/usr/local/bin/inews -h"	/* news posting cmd */
+/* #define SENDNEWS	"/usr/local/bin/inews -h" */
 
 
 /*--------- Program employed by users to change their password ---------*/
@@ -362,7 +245,7 @@ typedef unsigned short usign16_t;
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
 #define MAXPATH MAXPATHLEN      /* Longest pathname we ever expect */
-/* #define MAXPATH        (512)    /* Longest pathname we ever expect */
+/* #define MAXPATH        (512) */ /* Longest pathname we ever expect */
 #define MAXFOLDER      (64)     /* Longest foldername we ever expect */  
 
 
@@ -371,11 +254,6 @@ typedef unsigned short usign16_t;
 #define MAX_SCREEN_ROWS  (200) 
 
 
-/*---- When no screen size can be discovered this is the size used -----*/
-#define DEFAULT_LINES_ON_TERMINAL	(24)
-#define DEFAULT_COLUMNS_ON_TERMINAL	(80)
-
-
 /*----------------------------------------------------------------------
     Where to put the output of pine in debug mode. Files are created
  in the user's home directory and have a number appended to them when
@@ -414,7 +292,7 @@ typedef unsigned short usign16_t;
  ----*/
 #define MAX_FULLNAME     (100) 
 #define MAX_NICKNAME      (40)
-#define MAX_ADDRESS      (200)
+#define MAX_ADDRESS      (500)
 #define MAX_NEW_LIST     (500)  /* Max addrs to be added when creating list */
 #define MAX_SEARCH       (100)  /* Longest string to search for             */
 #define MAX_ADDR_EXPN   (1000)  /* Longest expanded addr                    */
