@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-lnx.h,v 1.20 1994/10/11 22:51:59 mikes Exp $
+  $Id: os-lnx.h,v 1.54 1998/10/07 21:51:21 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -14,28 +14,17 @@
 
    Please address all bugs and comments to "pine-bugs@cac.washington.edu"
 
-   Copyright 1989-1994  University of Washington
 
-    Permission to use, copy, modify, and distribute this software and its
-   documentation for any purpose and without fee to the University of
-   Washington is hereby granted, provided that the above copyright notice
-   appears in all copies and that both the above copyright notice and this
-   permission notice appear in supporting documentation, and that the name
-   of the University of Washington not be used in advertising or publicity
-   pertaining to distribution of the software without specific, written
-   prior permission.  This software is made available "as is", and
-   THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
-   WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
-   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
-   NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
-   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
-   (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
-   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-  
-   Pine and Pico are trademarks of the University of Washington.
-   No commercial use of these trademarks may be made without prior
-   written permission of the University of Washington.
+   Pine and Pico are registered trademarks of the University of Washington.
+   No commercial use of these trademarks may be made without prior written
+   permission of the University of Washington.
+
+   Pine, Pico, and Pilot software and its included text are Copyright
+   1989-1998 by the University of Washington.
+
+   The full text of our legal notices is contained in the file called
+   CPYRIGHT, included with this distribution.
+
 
    Pine is in part based on The Elm Mail System:
     ***********************************************************************
@@ -51,6 +40,8 @@
 #ifndef _OS_INCLUDED
 #define _OS_INCLUDED
 
+#include "../c-client/osdep.h"
+
 
 /*----------------------------------------------------------------------
 
@@ -69,17 +60,34 @@
    there should be a dummy disk_quota() routine in os-xxx.c so that defining
    this won't be harmful anyway.
  ----*/
-/* #define USE_QUOTAS  /* comment out if you never want quotas checked */
+/* #define USE_QUOTAS */
 
 
 
 /*----------------------------------------------------------------------
-   Define this if you want to allow the users to change their From header
-   line when they send out mail.  The users will still have to configure
-   either default-composer-hdrs or custumized-hdrs to get at the From
-   header, even if this is set.
+   Define this if you want to be sure to not allow users to change their
+   From header line when they send out mail. Even if you don't define
+   this the default is to not allow From to be changed. The user would have
+   to edit (by hand) "allow-changing-from" into their .pinerc feature list
+   or the sys-admin would have to include it in the default feature-list.
+   Even after that, the user still has to include it in either their
+   default-composer-hdrs or customized-hdrs to get at it. Instead of
+   defining NEVER_ALLOW_CHANGING_FROM, an easier way of preventing From
+   changing is to put the feature "no-allow-changing-from" in the
+   /usr/local/lib/pine.conf.fixed file.
  ----*/
-/* #define ALLOW_CHANGING_FROM  /* comment out to not allow changing From */
+/* #define NEVER_ALLOW_CHANGING_FROM */
+
+
+
+/*----------------------------------------------------------------------
+   Define this if you want to allow users to turn on the feature that
+   enables sending to take place in a fork()'d child.  This may reduce
+   the time on the user's wall clock it takes to post mail.
+   NOTE: You'll also have to make sure the appropriate osdep/postreap.*
+         file is included in the os-*.ic file for your system.
+ ----*/
+#define BACKGROUND_POST
 
 
 
@@ -91,8 +99,8 @@
 
 
 /*----------------------------------------------------------------------
-    Turn this on if it's OK to encode sent message text containing
-  "From " at the beginning of any line.
+    Turn this on to trigger QP encoding of sent message text if it contains
+  "From " at the beginning of a line or "." on a line by itself.
  ----*/
 /* #define ENCODE_FROMS */
 
@@ -101,9 +109,11 @@
 /*----------------------------------------------------------------------
     Timeouts (seconds)
  ----*/
-#define NEW_MAIL_TIME      (150) /* How often to check for new mail. There's 
-                                   some expense in doing this so it shouldn't
-                                   be done too frequently.  */
+#define DF_MAILCHECK      "150" /* How often to check for new mail, by
+				   default.  There's some expense in doing
+				   this so it shouldn't be done too
+				   frequently.  (Can be set in config
+				   file now.)  */
 
 /*----------------------------------------------------------------------
     Check pointing (seconds)
@@ -127,13 +137,31 @@
  In scrolling through text, the number of lines from the previous
  screen to overlap when showing the next screen.  Usually set to two.
  ----*/
-#define	OVERLAP		(2) 
+#define	DF_OVERLAP	"2"
+
+
+
+/*----------------------------------------------------------------------
+ When scrolling screens, the number of lines from top and bottom of
+ the screen to initiate single-line scrolling.
+ ----*/
+#define	DF_MARGIN	"0"
+
+
+
+/*----------------------------------------------------------------------
+ Default fill column for pine composer and maximum fill column.  The max
+ is used to stop people from setting their custom fill column higher than
+ that number.  Note that DF_FILLCOL is a string but MAX_FILLCOL is an integer.
+ ----*/
+#define	DF_FILLCOL	"74"
+#define	MAX_FILLCOL	80
 
 
 
 /*----- System-wide config file ----------------------------------------*/
-#define SYSTEM_PINERC             "/etc/pine.conf"
-#define SYSTEM_PINERC_FIXED       "/etc/pine.conf.fixed"
+#define SYSTEM_PINERC             "/usr/local/lib/pine.conf"
+#define SYSTEM_PINERC_FIXED       "/usr/local/lib/pine.conf.fixed"
 
 
 
@@ -146,6 +174,7 @@
 #define POSTPONED_MAIL            "postponed-mail"
 #define POSTPONED_MSGS            "postponed-msgs"
 #define INTERRUPTED_MAIL          ".pine-interrupted-mail"
+#define DEADLETTER                "dead.letter"
 #define DF_MAIL_DIRECTORY         "mail"
 #define INBOX_NAME                "INBOX"
 #define DF_SIGNATURE_FILE         ".signature"
@@ -157,6 +186,7 @@
 #define DF_SAVE_BY_SENDER         "no"
 #define DF_SORT_KEY               "arrival"
 #define DF_AB_SORT_RULE           "fullname-with-lists-last"
+#define DF_FLD_SORT_RULE          "alphabetical"
 #define DF_SAVED_MSG_NAME_RULE    "default-folder"
 #define DF_FCC_RULE               "default-fcc"
 #define DF_STANDARD_PRINTER       "lpr"
@@ -164,6 +194,14 @@
 #define DF_ADDRESSBOOK            ".addressbook"
 #define DF_BUGS_FULLNAME          "Pine Developers"
 #define DF_BUGS_ADDRESS           "pine-bugs@cac.washington.edu"
+#define DF_SUGGEST_FULLNAME       "Pine Developers"
+#define DF_SUGGEST_ADDRESS        "pine-suggestions@cac.washington.edu"
+#define DF_PINEINFO_FULLNAME      "Pine-Info News Group"
+#define DF_PINEINFO_ADDRESS       "pine-info@cac.washington.edu"
+#define DF_LOCAL_FULLNAME         "Local Support"
+#define DF_LOCAL_ADDRESS          "postmaster"
+#define DF_KBLOCK_PASSWD_COUNT    "1"
+#define DF_REMOTE_ABOOK_HISTORY   "3"
 
 /*----------------------------------------------------------------------
    The default printer when pine starts up for the first time with no printer
@@ -172,131 +210,27 @@
 
 
 
-/*----------------------------------------------------------------------
-
-   OS dependencies, Linux 0.99.10 version.  See also the os-lnx.c files.
-   The following stuff may need to be changed for a new port, but once
-   the port is done, it won't change.  At the bottom of the file are a few
-   constants that you may want to configure differently than they
-   are configured, but probably not.
-
- ----*/
-
-
-
-/*----------------- Are we ANSI? ---------------------------------------*/
-#define ANSI          /* this is an ANSI compiler */
-
-/*------ If our compiler doesn't understand type void ------------------*/
-/* #define void char     /* no void in compiler */
-
-/*-------- Standard ANSI functions usually defined in stdlib.h ---------*/
-#include <stdlib.h>  /* declarations of many common functions */
-/*---- Declare getenv() if not already declared ------------------------*/
-/* char *getenv();      /* often declared in stdlib.h */
-/* FILE *tmpfile(); */
-/* extern int errno; */
-/* Provide these two functions in os.c */
-
-/* int fgetpos(); int fsetpos(); */
-
-
-/*------- Standard string functions ------------------------------------*/
-#include <string.h>
-/*---- Declare these if not already declared ---------------------------*/
-/* char *strcpy(), *strncpy();  /* usually in string.h */
-
-
-/*------- Include types.h if you have it -------------------------------*/
-#include <sys/types.h>
-
-
-/*------- Some more includes that should usually be correct ------------*/
-#include <pwd.h>
-#include <sys/wait.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <netdb.h>
-
-
-
-/*----------------- time.h ---------------------------------------------*/
-#include <time.h> /* */
-/* plain time.h isn't enough on some systems */
-#include <sys/time.h>  /* For struct timeval usually in time.h */ 
-
-
-
-/*--------------- signal.h ---------------------------------------------*/
-#include <signal.h>      /* sometimes both required, sometimes */
-/* #include <sys/signal.h>  /* only one or the other */
-
-#define SIGSYS SIGUNUSED
-#define SIGEMT SIGUNUSED
-
-#define SigType void     /* value returned by signal() is void * */
-/* #define SigType int      /* value returned by signal() is int * */
-
-/* #define POSIX_SIGNALS    /* use POSIX signal semantics (ttyin.c) */
-
-/*--- For AT&T-derviced systems ----------------------------------------*/
-/* #define signal(a,b) sigset(a, b) /* sigset is just like signal */
-/* extern void (*sigset())();       /* declare sigset if not declared already */
-
-
-
-/*-------------- qsort argument type -----------------------------------*/
-#define QSType void  /* qsort arg is of type void * */
-/* #define QSType char  /* qsort arg is of type char * */
-
-
-
-/*------ how help text is referenced (always char ** on Unix) ----------*/
-#define HelpType char **
-#define NO_HELP (char **)NULL
-
-
-/*--------- System V terminal driver -----------------------------------*/
-#define HAVE_TERMIO     /* this is for pure System V */
-/* #define HAVE_TERMIOS    /* this is an alternative */
-/* #define USE_POLL        /* use the poll() system call instead of select() */
-/* #define USE_TERMINFO    /* use terminfo instead of termcap */
-/* #include <sys/stream.h> /* for ptem.h */
-/* #include <sys/ptem.h>   /* for struct winsize */
-
-
-
-/*-- What argument does wait(2) take? Define this if it is a union -----*/
-#define HAVE_WAIT_UNION  /* the arg to wait is a union wait * */
-
-
-/*-------- Is window resizing available? -------------------------------*/
-#define RESIZING  /* SIGWINCH and friends */
-
-
-
-/*-------- If no vfork, use regular fork -------------------------------*/
-/* #define vfork fork  /* vfork is just a lightweight fork, so can use fork */
-
-
-
 /*----- The usual sendmail configuration for sending mail on Unix ------*/
-#define SENDMAIL	"/usr/lib/sendmail"
-#define SENDMAILFLAGS	"-oi -oem -t"	/* ignore dots in incoming mail,
-					   mail back errors, and
-					   get recipients from To, cc,
-					   and bcc header lines.      */
+#define SENDMAIL	"/usr/sbin/sendmail"
+#define SENDMAILFLAGS	"-bs -odb -oem"	/* send via smtp with backgroud
+					   delivery and mail back errors */
+
+
+/*----------------------------------------------------------------------
+   If no nntp-servers are defined, this program will be used to post news.
+ ----*/
+/* #define SENDNEWS	"/usr/local/bin/inews -h" */
 
 
 /*--------- Program employed by users to change their password ---------*/
-#define	PASSWD_PROG	"/usr/bin/passwd"
+#define	PASSWD_PROG	"/bin/passwd"
 
 
 /*-------------- A couple constants used to size arrays ----------------*/
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
 #define MAXPATH MAXPATHLEN      /* Longest pathname we ever expect */
-/* #define MAXPATH     (512)    /* Longest pathname we ever expect */
+/* #define MAXPATH        (512) */ /* Longest pathname we ever expect */
 #define MAXFOLDER      (64)     /* Longest foldername we ever expect */  
 
 
@@ -305,11 +239,6 @@
 #define MAX_SCREEN_ROWS  (200) 
 
 
-/*---- When no screen size can be discovered this is the size used -----*/
-#define DEFAULT_LINES_ON_TERMINAL	(24)
-#define DEFAULT_COLUMNS_ON_TERMINAL	(80)
-
-
 /*----------------------------------------------------------------------
     Where to put the output of pine in debug mode. Files are created
  in the user's home directory and have a number appended to them when
@@ -341,14 +270,14 @@
  ----*/
 #define DEFAULT_DEBUG 2
 
-#define LPASS8 0x0800
+
 
 /*----------------------------------------------------------------------
     Various maximum field lengths, probably shouldn't be changed.
  ----*/
 #define MAX_FULLNAME     (100) 
 #define MAX_NICKNAME      (40)
-#define MAX_ADDRESS      (200)
+#define MAX_ADDRESS      (500)
 #define MAX_NEW_LIST     (500)  /* Max addrs to be added when creating list */
 #define MAX_SEARCH       (100)  /* Longest string to search for             */
 #define MAX_ADDR_EXPN   (1000)  /* Longest expanded addr                    */
