@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pine.c,v 4.457 1999/01/29 17:58:51 hubert Exp $";
+static char rcsid[] = "$Id: pine.c,v 4.481 1999/09/20 23:17:17 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -105,6 +105,7 @@ void	imap_telemetry_on PROTO((void));
 void	imap_telemetry_off PROTO((void));
 char   *pcpine_help_main PROTO((char *));
 int	pcpine_resize_main PROTO((void));
+int	pcpine_main_cursor PROTO((int, long));
 #endif
 
 
@@ -133,7 +134,7 @@ static struct key main_keys[] =
 	{"J","Journal",{MC_JOURNAL,1,{'j'}},KS_REVIEW},
 	{"S","Setup",{MC_SETUP,1,{'s'}},KS_NONE},
 	{"A","AddrBook",{MC_ADDRBOOK,1,{'a'}},KS_ADDRBOOK},
-	{"#","Role",{MC_ROLE,1,{'#'}},KS_NONE},
+	RCOMPOSE_MENU,
 	NULL_MENU};
 INST_KEY_MENU(main_keymenu, main_keys);
 #define MAIN_HELP_KEY		0
@@ -199,7 +200,8 @@ main(argc, argv)
     ps_global->sort_types[5]   = SortDate;
     ps_global->sort_types[6]   = SortSize;
     ps_global->sort_types[7]   = SortSubject2;
-    ps_global->sort_types[8]   = EndofList;
+    ps_global->sort_types[8]   = SortScore;
+    ps_global->sort_types[9]   = EndofList;
     ps_global->atmts           = (ATTACH_S *) fs_get(sizeof(ATTACH_S));
     ps_global->atmts_allocated = 1;
     ps_global->atmts->description = NULL;
@@ -356,11 +358,9 @@ main(argc, argv)
 	&& ((rv = atoi(pine_state->VAR_RSHOPENTIMEO)) == 0 || rv > 4))
       mail_parameters(NULL, SET_RSHTIMEOUT, (void *) rv);
 
-    /*
-     * Set up c-client call back to tell us about IMAP envelope arrivals
-     */
-    if(F_OFF(F_QUELL_IMAP_ENV_CB, ps_global))
-      mail_parameters(NULL, SET_IMAPENVELOPE, (void *) pine_imap_envelope);
+    if(pine_state->VAR_SSHOPENTIMEO
+	&& ((rv = atoi(pine_state->VAR_SSHOPENTIMEO)) == 0 || rv > 4))
+      mail_parameters(NULL, SET_SSHTIMEOUT, (void *) rv);
 
     /*
      * Tell c-client not to be so aggressive about uid mappings
@@ -393,6 +393,20 @@ main(argc, argv)
 	  }
     }
 
+    /*
+     * Install handler to let us know about potential delays
+     */
+    (void) mail_parameters(NULL, SET_BLOCKNOTIFY, (void *) pine_block_notify);
+
+    /*
+     * Install extra headers to fetch along with all the other stuff
+     * mail_fetch_structure and mail_fetch_overview requests.
+     */
+    calc_extra_hdrs();
+    if(get_extra_hdrs())
+      (void) mail_parameters(NULL, SET_IMAPEXTRAHEADERS,
+			     (void *) get_extra_hdrs());
+
     if(init_username(pine_state) < 0)
       exit(-1);
 
@@ -609,10 +623,6 @@ main(argc, argv)
 	  memset(&fcc, 0, sizeof(fcc));
 
 	if(good_addr){
-#if	defined(DOS) || defined(OS2)
-	    pine_state->permit_isolated_post = 1;
-#endif
-
 	    compose_mail(addr, fcc.tptr, NULL,
 			 args.data.mail.attachlist, stdin_getc);
 	}
@@ -1119,6 +1129,7 @@ main_menu_screen(pine_state)
 #ifdef	_WINDOWS
 	mswin_sethelptextcallback(pcpine_help_main);
 	mswin_setresizecallback(pcpine_resize_main);
+	mswin_mousetrackcallback(pcpine_main_cursor);
 #endif
 #endif
         ch = read_command();
@@ -1130,6 +1141,7 @@ main_menu_screen(pine_state)
 #ifdef	_WINDOWS
 	mswin_sethelptextcallback(NULL);
 	mswin_clearresizecallback(pcpine_resize_main);
+	mswin_mousetrackcallback(NULL);
 #endif
 #endif
 
@@ -1779,14 +1791,15 @@ setup_mini_menu(ql)
     char	*sigedit  = "Signature";
     char	*abooks   = "AddressBooks";
     char	*clctns   = "collectionList";
-    char	*roles    = "Roles";
+    char	*rules    = "Rules";
+    char	*color    = "Kolor";
 #ifdef	ENABLE_LDAP
     char	*dir      = "Directory";
 #endif
     HelpType     help     = h_mini_setup;
     int          deefault = 'p';
     int          s, ekey_num;
-    ESCKEY_S     setup_names[9];
+    ESCKEY_S     setup_names[10];
 
     ekey_num = 0;
 
@@ -1833,12 +1846,10 @@ setup_mini_menu(ql)
     setup_names[ekey_num].name    = "L";
     setup_names[ekey_num++].label = clctns;
 
-    if(F_OFF(F_DISABLE_ROLES_SETUP,ps_global)){ /* roles are allowed */
-	setup_names[ekey_num].ch      = 'r';
-	setup_names[ekey_num].rval    = 'r';
-	setup_names[ekey_num].name    = "R";
-	setup_names[ekey_num++].label = roles;
-    }
+    setup_names[ekey_num].ch      = 'r';
+    setup_names[ekey_num].rval    = 'r';
+    setup_names[ekey_num].name    = "R";
+    setup_names[ekey_num++].label = rules;
 
 #ifdef	ENABLE_LDAP
     setup_names[ekey_num].ch      = 'd';
@@ -1847,6 +1858,12 @@ setup_mini_menu(ql)
     setup_names[ekey_num++].label = dir;
 #endif
 
+    /* color */
+    setup_names[ekey_num].ch      = 'k';
+    setup_names[ekey_num].rval    = 'k';
+    setup_names[ekey_num].name    = "K";
+    setup_names[ekey_num++].label = color;
+
     setup_names[ekey_num].ch    = -1;
 
     if(F_ON(F_BLANK_KEYMENU,ps_global)){
@@ -1888,6 +1905,7 @@ do_setup_task(command)
     int command;
 {
     char *err = NULL;
+    int   rtype;
 
     switch(command) {
         /*----- EDIT SIGNATURE -----*/
@@ -1928,9 +1946,31 @@ do_setup_task(command)
 	ps_global->mangled_screen = 1;
 	break;
 
-        /*----- ROLES -----*/
+        /*----- RULES -----*/
       case 'r':
-	role_config_screen(ps_global);
+	rtype = rule_setup_type(ps_global, 0, "Type of rule setup : ");
+	switch(rtype){
+	  case 'r':
+	  case 's':
+	  case 'i':
+	  case 'f':
+	    role_config_screen(ps_global, (rtype == 'r') ? ROLE_DO_ROLES :
+					   (rtype == 's') ? ROLE_DO_SCORES :
+					    (rtype == 'f') ? ROLE_DO_FILTER :
+							      ROLE_DO_INCOLS);
+	    break;
+
+	  default:
+	    cmd_cancelled(NULL);
+	    break;
+	}
+
+	ps_global->mangled_screen = 1;
+	break;
+
+        /*----- COLOR -----*/
+      case 'k':
+	color_config_screen(ps_global);
 	ps_global->mangled_screen = 1;
 	break;
 
@@ -1967,6 +2007,66 @@ do_setup_task(command)
 }
 
 
+int
+rule_setup_type(ps, include_addr_opt, prompt)
+    struct pine *ps;
+    int          include_addr_opt;
+    char        *prompt;
+{
+    ESCKEY_S opts[6];
+    int ekey_num = 0, deefault = 0;
+
+    if(include_addr_opt){
+	deefault = 'a';
+	opts[ekey_num].ch      = 'a';
+	opts[ekey_num].rval    = 'a';
+	opts[ekey_num].name    = "A";
+	opts[ekey_num++].label = "Addrbook";
+    }
+
+    if(F_OFF(F_DISABLE_ROLES_SETUP,ps)){ /* roles are allowed */
+	if(deefault != 'a')
+	  deefault = 'r';
+
+	opts[ekey_num].ch      = 'r';
+	opts[ekey_num].rval    = 'r';
+	opts[ekey_num].name    = "R";
+	opts[ekey_num++].label = "Roles";
+    }
+    else if(deefault != 'a')
+      deefault = 's';
+
+    opts[ekey_num].ch      = 's';
+    opts[ekey_num].rval    = 's';
+    opts[ekey_num].name    = "S";
+    opts[ekey_num++].label = "SetScores";
+
+#ifndef	_WINDOWS
+    if(ps->color_style != COL_NONE && pico_hascolor()){
+#endif
+	if(deefault != 'a')
+	  deefault = 'i';
+
+	opts[ekey_num].ch      = 'i';
+	opts[ekey_num].rval    = 'i';
+	opts[ekey_num].name    = "I";
+	opts[ekey_num++].label = "Indexcolor";
+#ifndef	_WINDOWS
+    }
+#endif
+
+    opts[ekey_num].ch      = 'f';
+    opts[ekey_num].rval    = 'f';
+    opts[ekey_num].name    = "F";
+    opts[ekey_num++].label = "Filters";
+
+    opts[ekey_num].ch    = -1;
+
+    return(radio_buttons(prompt, -FOOTER_ROWS(ps), opts,
+			 deefault, 'x', NO_HELP, RB_NORM));
+}
+
+
 /*
  * Make sure any errors during initialization get queued for display
  */
@@ -2054,7 +2154,9 @@ new_user_or_version(ps)
 			  && !var_in_pinerc(ps->vars[V_NNTP_SERVER].name))
 		      || !var_in_pinerc(ps->vars[V_MAILCHECK].name)  /* 3.92 */
 		      || !var_in_pinerc(ps->vars[V_BROWSER].name)    /* 4.00 */
-		      || !var_in_pinerc(ps->vars[V_PATTERNS].name)); /* 4.10 */
+		      || !var_in_pinerc(ps->vars[V_PATTERNS].name)   /* 4.10 */
+								     /*4.20*/
+		      || !var_in_pinerc(ps->vars[V_TITLE_FORE_COLOR].name));
 
 
     /*
@@ -2403,6 +2505,8 @@ goodnight_gracey(pine_state, exit_val)
     clear_index_cache();
     free_newsgrp_cache();
     mailcap_free();
+    close_patterns();
+    free_extra_hdrs();
     free_contexts(&ps_global->context_list);
 #ifdef	ENABLE_LDAP
     free_saved_query_parameters();
@@ -2433,6 +2537,9 @@ goodnight_gracey(pine_state, exit_val)
       fs_give((void **)&pine_state->pine_dir);
 #endif
 
+    if(ps_global->hdr_colors)
+      free_hdr_colors(&ps_global->hdr_colors);
+
     if(ps_global->atmts){
 	for(i = 0; ps_global->atmts[i].description; i++){
 	    fs_give((void **)&ps_global->atmts[i].description);
@@ -2594,7 +2701,8 @@ flag_search(stream, flags, set_start, set_msgmap, ping)
 	}
     }
 
-    if(!is_imap_stream(stream) || modern_imap_stream(stream)){
+    if((!is_imap_stream(stream) || modern_imap_stream(stream))
+       && !(IS_NEWS(stream))){
 	pgm = mail_newsearchpgm();
 
 	if(flags & F_SEEN)
@@ -2729,9 +2837,10 @@ flag_search(stream, flags, set_start, set_msgmap, ping)
 	for(i = 1L; i <= stream->nmsgs; i++)
 	  mail_elt(stream, i)->searched = 0;
 
-	seq = build_sequence(stream, NULL, NULL);
-	mail_fetch_flags(stream, seq, 0L);
-	fs_give((void **) &seq);
+	if(seq = build_sequence(stream, NULL, NULL)){
+	    mail_fetch_flags(stream, seq, 0L);
+	    fs_give((void **) &seq);
+	}
     }
 }
 
@@ -2766,10 +2875,9 @@ count_flagged(stream, flags)
 
     /* Paw thru once more since all should be updated */
     for(n = 1L, count = 0L; n <= stream->nmsgs; n++)
-      if((mc = mail_elt(stream, n))->searched
-	  || (mc->valid
-	      && !get_lflag(stream, NULL, n, MN_EXLD)
-	      && FLAG_MATCH(flags, mc))){
+      if(((mc = mail_elt(stream, n))->searched
+	  || (mc->valid && FLAG_MATCH(flags, mc)))
+	 && !get_lflag(stream, NULL, n, MN_EXLD)){	  
 	  mc->searched = 1;	/* caller may be interested! */
 	  count++;
       }
@@ -3002,7 +3110,8 @@ same_stream(name, stream)
 	    : (strcmp(imap_host(stream), ".NO-IMAP-CONNECTION.")
 	       && ((*mb_n.user && !strcmp(mb_n.user, mb_s.user))
 		   || (!*mb_n.user
-		       && !strcmp(ps_global->VAR_USER_ID, mb_s.user))))))
+		       && !strcmp(ps_global->VAR_USER_ID, mb_s.user))
+		   || (!(*mb_n.user || *mb_s.user) && stream->anonymous)))))
       return(stream);
 
     dprint(3, (debugfile, "same_stream: no dice\n"));
@@ -3105,6 +3214,36 @@ panic1(message, arg)
 }
 
 
+/*----------------------------------------------------------------------
+  Call back for c-client to feed us back the progress of network reads
+
+  Input: 
+
+ Result: 
+  ----*/
+MAILSTREAM *
+pine_mail_open(stream, mailbox, flags)
+    MAILSTREAM *stream;
+    char       *mailbox;
+    long	flags;
+{
+#ifdef	DEBUG
+    if(ps_global->debug_imap > 3)
+      flags |= OP_DEBUG;
+#endif
+    
+    if(F_ON(F_PREFER_ALT_AUTH, ps_global)){
+	DRIVER *d;
+
+	if((d = mail_valid (NIL,mailbox, (char *) NIL))
+	   && !strcmp(d->name, "imap"))
+	  flags |= OP_TRYALT;
+    }
+
+    return(mail_open(stream, mailbox, flags));
+}
+
+
 
 /*----------------------------------------------------------------------
   Call back for c-client to feed us back the progress of network reads
@@ -3281,4 +3420,22 @@ pcpine_resize_main()
     (void) get_windsize (ps_global->ttyo);
     main_redrawer();
 }
-#endif
+
+
+int
+pcpine_main_cursor(col, row)
+    int  col;
+    long row;
+{
+    unsigned char ndmi;
+
+    ndmi = (unsigned char) row - 3;
+    if(row >= 3 && !(ndmi & 0x01)
+       && ndmi <= (unsigned) MAX_DEFAULT_MENU_ITEM
+       && ndmi < ps_global->ttyo->screen_rows - 4 - FOOTER_ROWS(ps_global)
+       && !(ndmi & 0x01))
+      return(MSWIN_CURSOR_HAND);
+    else
+      return(MSWIN_CURSOR_ARROW);
+}
+#endif /* _WINDOWS */
