@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.108 1999/01/13 02:51:34 mikes Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.126 1999/09/08 21:44:26 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: screen.c,v 4.108 1999/01/13 02:51:34 mikes Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1998 by the University of Washington.
+   1989-1999 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -60,6 +60,7 @@ int   equalbits PROTO((bitmap_t, bitmap_t, int));
 char *percentage PROTO((long, long, int));
 void  output_keymenu PROTO((struct key_menu *, bitmap_t, int, int));
 int   digit_count PROTO((long));
+void  output_titlebar PROTO((char *, int));
 #ifdef	MOUSE
 void  print_inverted_label PROTO((int, MENUITEM *));
 #endif
@@ -128,9 +129,10 @@ int		 row,
 			 real_row,
 			 max_column, /* number of columns on screen */
 			 off;        /* offset into keymap */
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR          *lastc=NULL, *label_color=NULL, *name_color=NULL;
 #ifdef	MOUSE
     char		 keystr[MAX_KEYNAME + MAX_LABEL + 2];
-    extern void          register_key();
 #endif
 
     off    	  = km->which * 12;
@@ -143,6 +145,25 @@ int		 row,
 
     real_row = row > 0 ? row : ps_global->ttyo->screen_rows + row;
 
+    if(pico_usingcolor()){
+	lastc = pico_get_cur_color();
+	if(lastc && VAR_KEYLABEL_FORE_COLOR && VAR_KEYLABEL_BACK_COLOR &&
+	   pico_is_good_color(VAR_KEYLABEL_FORE_COLOR) &&
+	   pico_is_good_color(VAR_KEYLABEL_BACK_COLOR)){
+	    label_color = new_color_pair(VAR_KEYLABEL_FORE_COLOR,
+					 VAR_KEYLABEL_BACK_COLOR);
+	    if(label_color)
+	      (void)pico_set_colorp(label_color, PSC_NONE);
+	}
+
+	if(label_color && VAR_KEYNAME_FORE_COLOR && VAR_KEYNAME_BACK_COLOR &&
+	   pico_is_good_color(VAR_KEYNAME_FORE_COLOR) &&
+	   pico_is_good_color(VAR_KEYNAME_BACK_COLOR)){
+	    name_color = new_color_pair(VAR_KEYNAME_FORE_COLOR,
+					VAR_KEYNAME_BACK_COLOR);
+	}
+    }
+
     if(keymenu_is_dirty){
 	ClearLines(real_row, real_row+1);
 	keymenu_is_dirty = 0;
@@ -171,7 +192,7 @@ int		 row,
 		   : km->keys[e].label == NULL ? "(null)" 
 		   : km->keys[e].label, km ? km->keys[e].column : 0));
 #ifdef	MOUSE
-	register_key(i, NO_OP_COMMAND, "", NULL, 0, 0, 0);
+	register_key(i, NO_OP_COMMAND, "", NULL, 0, 0, 0, NULL, NULL);
 #endif
     }
 
@@ -186,9 +207,11 @@ int		 row,
 	char  this_label[MAX_LABEL+1];
 
 	j = 6*i - 1;
-	if(i == 1)
-	  max_column--;  /* some terminals scroll if you write in the
-			    lower right hand corner */
+	if(i == 1 && !label_color)
+	  max_column--;  /* Some terminals scroll if you write in the
+			    lower right hand corner. If user has a
+			    label_color set we'll take our chances.
+			    Otherwise, we'd get one cell of Normal. */
 
 	/*
 	 * k is the key struct we're working on
@@ -288,13 +311,21 @@ int		 row,
 		    strcpy(keystr, temp);
 #endif
 		    MoveCursor(real_row + i, column + k->column);
-		    if(!empty)
-		      StartInverse();
+		    if(!empty){
+			if(name_color)
+			  (void)pico_set_colorp(name_color, PSC_NONE);
+			else
+			  StartInverse();
+		    }
 
 		    Write_to_screen(temp);
 		    c = k->column + strlen(temp);
-		    if(!empty)
-		      EndInverse();
+		    if(!empty){
+			if(name_color)
+			  (void)pico_set_colorp(label_color, PSC_NONE);
+			else
+			  EndInverse();
+		    }
 
 		    /* now the space after the name and the label */
 		    temp[0] = '\0';
@@ -357,7 +388,8 @@ int		 row,
 						    ? SPACE
 						    : k->name[0],
 			   keystr, print_inverted_label,
-			   real_row+i, k->column, len);
+			   real_row+i, k->column, len, 
+			   name_color, label_color);
 	    }
 #endif
 
@@ -368,6 +400,14 @@ int		 row,
     }
 
     fflush(stdout);
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+	if(label_color)
+	  free_color_pair(&label_color);
+	if(name_color)
+	  free_color_pair(&name_color);
+    }
 }
 
 
@@ -375,6 +415,11 @@ int		 row,
 /*
  * print_inverted_label - highlight the label of the given menu item.
  * (callback from pico mouse routines)
+ *
+ * This stuff's a little strange because the row range is inclusive but
+ * the column range excludes the right hand edge. So far, this is only
+ * ever called with the top left row equal to the bottom right row.
+ * If you change that you probably need to fix it.
  */
 void
 print_inverted_label(state, m)
@@ -382,34 +427,122 @@ print_inverted_label(state, m)
     MENUITEM *m;
 {
     unsigned i, j;
-    int      col_offset;
-    char    *lp;
+    int      col_offset, do_color = 0, skip = 0, len, c;
+    char    *lp, *label;
+    COLOR_PAIR *name_color = NULL, *label_color = NULL, *lastc = NULL;
+    COLOR_PAIR *defnamec = NULL, *deflabelc = NULL;
+    struct variable *vars = ps_global->vars;
 
-    /*
-     * Leave the command name bold
-     */
-    col_offset = (state || !(lp=strchr(m->label, ' '))) ? 0 : (lp - m->label);
-    MoveCursor((int)(m->tl.r), (int)(m->tl.c) + col_offset);
-    if(state)
-      StartInverse();
-    else
-      EndInverse();
+    col_offset = (m->label && (lp=strchr(m->label, ' '))) ? (lp - m->label) : 0;
+    if(pico_usingcolor() && ((VAR_KEYLABEL_FORE_COLOR &&
+			      VAR_KEYLABEL_BACK_COLOR) ||
+			     (VAR_KEYNAME_FORE_COLOR &&
+			      VAR_KEYNAME_BACK_COLOR))){
+	lastc = pico_get_cur_color();
 
-    for(i = m->tl.r; i <= m->br.r; i++)
-      for(j = m->tl.c + col_offset; j <= m->br.c; j++)
-	if(i == m->lbl.r && j == m->lbl.c + col_offset && m->label){
-	    lp = m->label + col_offset;		/* show label?? */
-	    while(*lp && j++ < m->br.c)
-	      Writechar((unsigned int)(*lp++), 0);
-
-	    continue;
+	if(VAR_KEYNAME_FORE_COLOR && VAR_KEYNAME_BACK_COLOR){
+	    name_color = state ? new_color_pair(VAR_KEYNAME_BACK_COLOR,
+						VAR_KEYNAME_FORE_COLOR)
+			       : new_color_pair(VAR_KEYNAME_FORE_COLOR,
+						VAR_KEYNAME_BACK_COLOR);
 	}
+	else if(VAR_REV_FORE_COLOR && VAR_REV_BACK_COLOR)
+	  name_color = new_color_pair(VAR_REV_FORE_COLOR, VAR_REV_BACK_COLOR);
+
+	if(VAR_KEYLABEL_FORE_COLOR && VAR_KEYLABEL_BACK_COLOR){
+	    label_color = state ? new_color_pair(VAR_KEYLABEL_BACK_COLOR,
+						 VAR_KEYLABEL_FORE_COLOR)
+			        : new_color_pair(VAR_KEYLABEL_FORE_COLOR,
+						 VAR_KEYLABEL_BACK_COLOR);
+	}
+	else if(VAR_REV_FORE_COLOR && VAR_REV_BACK_COLOR){
+	    label_color = state ? new_color_pair(VAR_REV_FORE_COLOR,
+						 VAR_REV_BACK_COLOR)
+			        : new_color_pair(VAR_NORM_FORE_COLOR,
+						 VAR_NORM_BACK_COLOR);
+	}
+
+	/*
+	 * See if we can grok all these colors. If not, we're going to
+	 * punt and pretend there are no colors at all.
+	 */
+	if(!pico_is_good_colorpair(name_color) ||
+	   !pico_is_good_colorpair(label_color)){
+	    if(name_color)
+	      free_color_pair(&name_color);
+	    if(label_color)
+	      free_color_pair(&label_color);
+	    if(lastc)
+	      free_color_pair(&lastc);
+	}
+	else{
+	    do_color++;
+	    (void)pico_set_colorp(label_color, PSC_NONE);
+	    if(!(VAR_KEYLABEL_FORE_COLOR && VAR_KEYLABEL_BACK_COLOR)){
+		if(state)
+		  StartInverse();
+		else
+		  EndInverse();
+	    }
+	}
+    }
+
+    if(!do_color){
+	/*
+	 * Command name's already inverted, leave it.
+	 */
+	skip = state ? 0 : col_offset;
+	if(state)
+	  StartInverse();
 	else
-	  Writechar(' ', 0);
+	  EndInverse();
+    }
 
-    if(state)
-      EndInverse();			/* turn inverse back off */
+    MoveCursor((int)(m->tl.r), (int)(m->tl.c) + skip);
 
+    label = m->label ? m->label : "";
+    len = strlen(label);
+    for(i = m->tl.r; i <= m->br.r; i++)
+      for(j = m->tl.c + skip; j < m->br.c; j++){
+	  c = (i == m->lbl.r &&
+	       j - m->lbl.c >= 0 &&
+	       j - m->lbl.c < len) ? label[j - m->lbl.c] : ' ';
+	  if(name_color && col_offset && j == m->lbl.c)
+	    (void)pico_set_colorp(name_color, PSC_NONE);
+
+	  if(name_color && col_offset && j == m->lbl.c + col_offset){
+	      if(label_color)
+		(void)pico_set_colorp(label_color, PSC_NONE);
+	      else{
+		  if(state)
+		    StartInverse();
+		  else
+		    EndInverse();
+	      }
+	  }
+
+	  Writechar((unsigned int)c, 0);
+      }
+
+    if(do_color){
+	if(lastc){
+	    (void)pico_set_colorp(lastc, PSC_NONE);
+	    free_color_pair(&lastc);
+	}
+	else if(state)
+	  EndInverse();
+	else
+	  pico_set_normal_color();
+	
+	if(name_color)
+	  free_color_pair(&name_color);
+	if(label_color)
+	  free_color_pair(&label_color);
+    }
+    else{
+	if(state)
+	  EndInverse();
+    }
 }
 #endif	/* MOUSE */
 
@@ -421,15 +554,25 @@ void
 blank_keymenu(row, column)
 int row, column;
 {
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR *lastc;
+
     if(FOOTER_ROWS(ps_global) > 1){
 	km_state.blanked    = 1;
 	km_state.row        = row;
 	km_state.column     = column;
 	MoveCursor(row, column);
+	lastc = pico_set_colors(VAR_KEYLABEL_FORE_COLOR,
+				VAR_KEYLABEL_BACK_COLOR, PSC_NORM|PSC_RET);
+
 	CleartoEOLN();
 	MoveCursor(row+1, column);
 	CleartoEOLN();
 	fflush(stdout);
+	if(lastc){
+	    (void)pico_set_colorp(lastc, PSC_NONE);
+	    free_color_pair(&lastc);
+	}
     }
 }
 
@@ -825,9 +968,6 @@ redraw_keymenu()
 				  || F_ON(F_FAKE_NEW_IN_NEWS, ps_global)) \
 			      && ((X) & MS_NEW)) ? "NEW" : "   ")
 
-#define	BAR_NUMBER(X)
-
-
 
 static struct titlebar_state {
     MAILSTREAM	*stream;
@@ -1049,12 +1189,8 @@ set_titlebar(title, stream, cntxt, folder, msgmap, display_on_screen, style,
     }
 
     tb = format_titlebar(&start_col);
-    if(display_on_screen){
-        StartInverse();
-        PutLine0(0, start_col, tb+start_col);
-        EndInverse();
-        fflush(stdout);
-    }
+    if(display_on_screen)
+      output_titlebar(tb, start_col);
 
     return(tb);
 }
@@ -1065,14 +1201,31 @@ redraw_titlebar()
     int   start_col;
     char *tb;
 
-    StartInverse();
     tb = format_titlebar(&start_col);
+    output_titlebar(tb, start_col);
+}
+
+
+void
+output_titlebar(tb, start_col)
+    char *tb;
+    int   start_col;
+{
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR *lastc;
+
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
+	    
     PutLine0(0, start_col, tb+start_col);
-    EndInverse();
+
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
     fflush(stdout);
 }
-     
-
 
 
 /*----------------------------------------------------------------------
@@ -1306,13 +1459,21 @@ columns and shorten tail, the string holding the rest of the line.
 void
 update_titlebar_message()
 {
+    struct variable *vars = ps_global->vars;
+
     if(as.cur_mess_col >= 0 && as.current_msg != mn_get_cur(as.msgmap)){
+	COLOR_PAIR *lastc;
 	int delta = digit_count(mn_get_cur(as.msgmap))
 						 - digit_count(as.current_msg);
 	as.current_msg = mn_get_cur(as.msgmap);
-	StartInverse();
-	if(delta) {
+	lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+				PSC_REV|PSC_RET);
+
+	if(delta){
+	    as.del_column += delta;
+
 	    if(as.style == MsgTextPercent){
+		as.percent_column += delta;
 		PutLine5(0, as.cur_mess_col, "%s of %s %s %s%s",
 			 strcpy(tmp_20k_buf + 1000, comatose(as.current_msg)),
 			 strcpy(tmp_20k_buf + 1500,
@@ -1320,23 +1481,23 @@ update_titlebar_message()
 			 percentage(as.current_line, as.total_lines, 0),
 			 BAR_STATUS(as.msg_state),
 			 repeat_char(max(0, -delta), SPACE));
-		as.del_column     += delta;
-		as.percent_column += delta;
-	    }
-	    else{
-		PutLine4(0, as.cur_mess_col, "%s of %s %s%s",
-			 strcpy(tmp_20k_buf + 1000, comatose(as.current_msg)),
-			 strcpy(tmp_20k_buf + 1500,
-				comatose(mn_get_total(as.msgmap))),
-			 BAR_STATUS(as.msg_state),
-			 repeat_char(max(0, -delta), SPACE));
-		as.del_column += delta;
 	    }
+	    else
+	      PutLine4(0, as.cur_mess_col, "%s of %s %s%s",
+		       strcpy(tmp_20k_buf + 1000, comatose(as.current_msg)),
+		       strcpy(tmp_20k_buf + 1500,
+			      comatose(mn_get_total(as.msgmap))),
+		       BAR_STATUS(as.msg_state),
+		       repeat_char(max(0, -delta), SPACE));
 	}
 	else
 	  PutLine0(0, as.cur_mess_col, comatose(as.current_msg));
 
-	EndInverse();
+	if(lastc){
+	    (void)pico_set_colorp(lastc, PSC_NONE);
+	    free_color_pair(&lastc);
+	}
+
 	fflush(stdout);
     }
 }
@@ -1349,42 +1510,53 @@ update_titlebar_message()
   Args:  None, operates on state set during most recent set_titlebar call
 
   ---*/
-void
+int
 update_titlebar_status()
 {
     MESSAGECACHE *mc;
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR  *lastc;
     
     if(!as.stream || as.current_msg < 0L || as.del_column < 0)
-      return;
+      return(1);
 
     mc = mail_elt(as.stream, mn_m2raw(as.msgmap, as.current_msg));
 
+    if(!mc->valid)
+      return(0);			/* indeterminate */
+
     if(mc->deleted){			/* deleted takes precedence */
 	if(as.msg_state & MS_DEL)
-	  return;
+	  return(1);
     }
     else if(mc->answered){		/* then answered */
 	if(as.msg_state & MS_ANS)
-	  return;
+	  return(1);
     }
     else if(!mc->seen && as.stream
 	    && (!IS_NEWS(as.stream)
 		|| (mc->recent && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)))){
 	if(as.msg_state & MS_NEW)	/* then seen */
-	  return;
+	  return(1);
     }
-    else if(IS_NEWS(as.stream) && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)){
-    }
-    else if(as.msg_state == 0)		/* nothing to change... */
-      return;
+    else if(!(IS_NEWS(as.stream) && F_ON(F_FAKE_NEW_IN_NEWS, ps_global))
+	    && as.msg_state == 0)		/* nothing to change... */
+      return(1);
 
     as.msg_state = STATUS_BITS(mc);
-    StartInverse();
-    PutLine0(0, as.del_column, BAR_STATUS(as.msg_state));
-    EndInverse();
-    fflush(stdout);
-}
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
 
+    PutLine0(0, as.del_column, BAR_STATUS(as.msg_state));
+
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
+    fflush(stdout);
+    return(1);
+}
 
 
 /*---------------------------------------------------------------------- 
@@ -1398,13 +1570,22 @@ void
 update_titlebar_percent(line)
     long line;
 {
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR *lastc;
+
     if(as.percent_column < 0)
       return;
 
-    StartInverse();
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
+
     PutLine0(0, as.percent_column,
 	     percentage(as.current_line = line, as.total_lines, 0));
-    EndInverse();
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
     fflush(stdout);
 }
 
@@ -1421,6 +1602,9 @@ void
 update_titlebar_lpercent(new_line_number)
     long new_line_number;
 {
+    struct variable *vars = ps_global->vars;
+    COLOR_PAIR *lastc;
+
     if(as.page_column < 0 || new_line_number == as.current_line)
       return;
 
@@ -1430,9 +1614,15 @@ update_titlebar_lpercent(new_line_number)
 	    digit_count(as.total_lines), as.current_line, 
 	    digit_count(as.total_lines), as.total_lines,
 	    percentage(as.current_line, as.total_lines, 0));
-    StartInverse();
+    lastc = pico_set_colors(VAR_TITLE_FORE_COLOR, VAR_TITLE_BACK_COLOR,
+			    PSC_REV|PSC_RET);
+
     PutLine0(0, as.page_column, tmp_20k_buf);
-    EndInverse();
+    if(lastc){
+	(void)pico_set_colorp(lastc, PSC_NONE);
+	free_color_pair(&lastc);
+    }
+
     fflush(stdout);
 }
 
