@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signals.c,v 4.129 1999/04/21 00:24:37 mikes Exp $";
+static char rcsid[] = "$Id: signals.c,v 4.130 1999/11/09 23:56:19 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -1148,7 +1148,7 @@ mm_critical(stream)
     hold_int  = signal(SIGINT, SIG_IGN);
     hold_term = signal(SIGTERM, SIG_IGN);
     dprint(9, (debugfile, "Done with IMAP critical on %s\n",
-              stream ? stream->mailbox : "<no folder>" ));
+              (stream && stream->mailbox) ? stream->mailbox : "<no folder>" ));
 }
 
 
@@ -1169,7 +1169,7 @@ mm_nocritical(stream)
     (void)signal(SIGINT, hold_int);
     (void)signal(SIGTERM, hold_term);
     dprint(9, (debugfile, "Done with IMAP critical on %s\n",
-              stream ? stream->mailbox : "<no folder>" ));
+              (stream && stream->mailbox) ? stream->mailbox : "<no folder>" ));
 }
 
 
